package org.jeecg.config;

import org.apache.shiro.mgt.DefaultSessionStorageEvaluator;
import org.apache.shiro.mgt.DefaultSubjectDAO;
import org.apache.shiro.mgt.SecurityManager;
import org.apache.shiro.spring.LifecycleBeanPostProcessor;
import org.apache.shiro.spring.security.interceptor.AuthorizationAttributeSourceAdvisor;
import org.apache.shiro.spring.web.ShiroFilterFactoryBean;
import org.apache.shiro.web.mgt.DefaultWebSecurityManager;
import org.jeecg.modules.shiro.authc.ShiroRealm;
import org.jeecg.modules.shiro.authc.aop.JwtFilter;
import org.springframework.aop.framework.autoproxy.DefaultAdvisorAutoProxyCreator;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.DependsOn;

import javax.servlet.Filter;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * @author: Scott
 * @date: 2018/2/7
 * @description: shiro 配置类
 */

@Configuration
public class ShiroConfig {

	/**
	 * Filter Chain定义说明
	 *
	 * 1、一个URL可以配置多个Filter，使用逗号分隔
	 * 2、当设置多个过滤器时，全部验证通过，才视为通过
	 * 3、部分过滤器可指定参数，如perms，roles
	 */
	@Bean("shiroFilter")
	public ShiroFilterFactoryBean shiroFilter(SecurityManager securityManager) {
		ShiroFilterFactoryBean shiroFilterFactoryBean = new ShiroFilterFactoryBean();
		shiroFilterFactoryBean.setSecurityManager(securityManager);
		// 拦截器
		Map<String, String> filterChainDefinitionMap = new LinkedHashMap<String, String>();
		// 配置不会被拦截的链接 顺序判断
        //登录接口排除
        filterChainDefinitionMap.put("/sys/login", "anon");
        filterChainDefinitionMap.put("/sys/wxlogin", "anon");

        //获取加密串
        filterChainDefinitionMap.put("/sys/getEncryptedString", "anon");
        //短信验证码
        filterChainDefinitionMap.put("/sys/sms", "anon");
        //手机登录
        filterChainDefinitionMap.put("/sys/phoneLogin", "anon");
        //校验用户是否存在
        filterChainDefinitionMap.put("/sys/user/checkOnlyUser", "anon");
        //用户注册
        filterChainDefinitionMap.put("/sys/user/register", "anon");
        //根据手机号获取用户信息
        filterChainDefinitionMap.put("/sys/user/querySysUser", "anon");
        //用户忘记密码验证手机号
        filterChainDefinitionMap.put("/sys/user/phoneVerification", "anon");
        //用户更改密码
        filterChainDefinitionMap.put("/sys/user/passwordChange", "anon");
        //登录验证码
        filterChainDefinitionMap.put("/auth/2step-code", "anon");
        //图片预览不限制token
        filterChainDefinitionMap.put("/sys/common/view/**", "anon");
        //文件下载不限制token
        filterChainDefinitionMap.put("/sys/common/download/**", "anon");
        //pdf预览
        filterChainDefinitionMap.put("/sys/common/pdf/**", "anon");
		filterChainDefinitionMap.put("/ctop/bytedanceAreaInfo/**", "anon");
		filterChainDefinitionMap.put("/ctop/bytedanceInterestCategory/**", "anon");
		filterChainDefinitionMap.put("/template/creative/words/get", "anon");
		filterChainDefinitionMap.put("/appium/crawler/*", "anon");

        filterChainDefinitionMap.put("/ctop/kuaishouChannelRegInfo/*", "anon");


		filterChainDefinitionMap.put("/ctop/byteDance/homePage/report/*", "anon");
		filterChainDefinitionMap.put("/ctop/material/top/cost", "anon");




		//优化师相关
		//filterChainDefinitionMap.put("/ctop/performanceOptimizer/detail", "anon");
		//filterChainDefinitionMap.put("/ctop/performanceOptimizer/ranking/list", "anon");
		//filterChainDefinitionMap.put("/ctop/performance/project/list", "anon");
		//filterChainDefinitionMap.put("/ctop/performance/account/list", "anon");
		//filterChainDefinitionMap.put("/ctop/optimizerRoleInfo/manager/detail", "anon");


        filterChainDefinitionMap.put("/generic/**", "anon");
		filterChainDefinitionMap.put("/", "anon");
		filterChainDefinitionMap.put("/doc.html", "anon");
		filterChainDefinitionMap.put("/**/*.js", "anon");
		filterChainDefinitionMap.put("/**/*.css", "anon");
		filterChainDefinitionMap.put("/**/*.html", "anon");
		filterChainDefinitionMap.put("/**/*.svg", "anon");
		filterChainDefinitionMap.put("/**/*.jpg", "anon");
		filterChainDefinitionMap.put("/**/*.png", "anon");
		filterChainDefinitionMap.put("/**/*.ico", "anon");
		filterChainDefinitionMap.put("/druid/**", "anon");
		filterChainDefinitionMap.put("/swagger-ui.html", "anon");
		filterChainDefinitionMap.put("/swagger**/**", "anon");
		filterChainDefinitionMap.put("/webjars/**", "anon");
		filterChainDefinitionMap.put("/v2/**", "anon");
        filterChainDefinitionMap.put("/kuaishou", "anon");
        filterChainDefinitionMap.put("/kuaishou/report", "anon");
		filterChainDefinitionMap.put("/ctop/bindAccount/**", "anon");
		filterChainDefinitionMap.put("/**/*.mp4", "anon");


		//性能监控
		filterChainDefinitionMap.put("/actuator/metrics/**", "anon");
		filterChainDefinitionMap.put("/actuator/httptrace/**", "anon");
		filterChainDefinitionMap.put("/actuator/redis/**", "anon");
        //消息队列
        filterChainDefinitionMap.put("/mq/**", "anon");
        //模板测试
        filterChainDefinitionMap.put("/test/**", "anon");
		//oauth接口
        filterChainDefinitionMap.put("/auth/**", "anon");
		filterChainDefinitionMap.put("/ctop/syncdata/**", "anon");
		//头条接口
        filterChainDefinitionMap.put("/bytedance", "anon");
		filterChainDefinitionMap.put("/toutiao/dictitem/list", "anon");
        filterChainDefinitionMap.put("/toutiao/loadAllData", "anon");
        filterChainDefinitionMap.put("/toutiao/loadSingleData", "anon");
		filterChainDefinitionMap.put("/kuaishou/getReport/historyData", "anon");
		filterChainDefinitionMap.put("/toutiao/advertiser/**", "anon");
        filterChainDefinitionMap.put("/toutiao/industry/list", "anon");
        filterChainDefinitionMap.put("/template/bytedance/industry/list", "anon");
        filterChainDefinitionMap.put("/template/convert/list", "anon");
		filterChainDefinitionMap.put("/template/convert/list/url", "anon");
        //文件上传接口
		filterChainDefinitionMap.put("/upload/**", "anon");
        filterChainDefinitionMap.put("/appium/appiumTask/run", "anon");

        filterChainDefinitionMap.put("/system/userCompany/**", "anon");
        filterChainDefinitionMap.put("/qywexin", "anon");
        filterChainDefinitionMap.put("/test/getGroupData/**", "anon");
        filterChainDefinitionMap.put("/test/getVideoAndImage/**", "anon");
        filterChainDefinitionMap.put("/test/gerCreative/**", "anon");

		filterChainDefinitionMap.put("/report/kuaishouReportDailyAgent/get","anon");

		filterChainDefinitionMap.put("/appium/appiumTask/run/login", "anon");
		filterChainDefinitionMap.put("/zip/download", "anon");
		//爬虫接口
		filterChainDefinitionMap.put("/graphql/video", "anon");
		filterChainDefinitionMap.put("/ks/web/test", "anon");
        filterChainDefinitionMap.put("/ureport/**", "anon");
        filterChainDefinitionMap.put("/ctop/advertiser/**", "anon");

		// 添加自己的过滤器并且取名为jwt
		Map<String, Filter> filterMap = new HashMap<>(1);
		filterMap.put("jwt", new JwtFilter());
		shiroFilterFactoryBean.setFilters(filterMap);
		// <!-- 过滤链定义，从上向下顺序执行，一般将/**放在最为下边
		filterChainDefinitionMap.put("/**", "jwt");

		// 未授权界面返回JSON
		shiroFilterFactoryBean.setUnauthorizedUrl("/sys/common/403");
		shiroFilterFactoryBean.setLoginUrl("/sys/common/403");
		shiroFilterFactoryBean.setFilterChainDefinitionMap(filterChainDefinitionMap);
		return shiroFilterFactoryBean;
	}

	@Bean("securityManager")
	public DefaultWebSecurityManager securityManager(ShiroRealm myRealm) {
		DefaultWebSecurityManager securityManager = new DefaultWebSecurityManager();
		securityManager.setRealm(myRealm);

		/*
		 * 关闭shiro自带的session，详情见文档
		 * http://shiro.apache.org/session-management.html#SessionManagement-
		 * StatelessApplications%28Sessionless%29
		 */
		DefaultSubjectDAO subjectDAO = new DefaultSubjectDAO();
		DefaultSessionStorageEvaluator defaultSessionStorageEvaluator = new DefaultSessionStorageEvaluator();
		defaultSessionStorageEvaluator.setSessionStorageEnabled(false);
		subjectDAO.setSessionStorageEvaluator(defaultSessionStorageEvaluator);
		securityManager.setSubjectDAO(subjectDAO);

		return securityManager;
	}

	/**
	 * 下面的代码是添加注解支持
	 * @return
	 */
	@Bean
	@DependsOn("lifecycleBeanPostProcessor")
	public DefaultAdvisorAutoProxyCreator defaultAdvisorAutoProxyCreator() {
		DefaultAdvisorAutoProxyCreator defaultAdvisorAutoProxyCreator = new DefaultAdvisorAutoProxyCreator();
		defaultAdvisorAutoProxyCreator.setProxyTargetClass(true);
		return defaultAdvisorAutoProxyCreator;
	}

	@Bean
	public LifecycleBeanPostProcessor lifecycleBeanPostProcessor() {
		return new LifecycleBeanPostProcessor();
	}

	@Bean
	public AuthorizationAttributeSourceAdvisor authorizationAttributeSourceAdvisor(DefaultWebSecurityManager securityManager) {
		AuthorizationAttributeSourceAdvisor advisor = new AuthorizationAttributeSourceAdvisor();
		advisor.setSecurityManager(securityManager);
		return advisor;
	}

}
