package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.UserAllocation;
import cn.com.ctop.common.module.service.ISysRoleService;
import cn.com.ctop.common.module.service.IUserAllocationService;
import cn.com.ctop.common.module.utils.*;
import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.metadata.Sheet;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.DateUtils;
import org.jeecg.modules.ctop.entity.Advertiser;
import org.jeecg.modules.ctop.entity.Project;
import org.jeecg.modules.ctop.mapper.ProjectMapper;
import org.jeecg.modules.ctop.mapper.ProjectMemberMapper;
import org.jeecg.modules.ctop.service.IAdvertiserService;
import org.jeecg.modules.ctop.service.IProjectService;
import org.jeecg.modules.ctop.vo.SheetInfoVo;
import org.jeecg.modules.excelutil.entity.KuaishouXxlEntity;
import org.jeecg.modules.system.entity.SysCategory;
import org.jeecg.modules.system.entity.SysUser;
import org.jeecg.modules.system.entity.UserCompany;
import org.jeecg.modules.system.mapper.UserCompanyMapper;
import org.jeecg.modules.system.service.ISysCategoryService;
import org.jeecg.modules.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.*;

/**
 * @Description: 广告主信息表
 * @Author: jeecg-boot
 * @Date: 2019-06-11
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "广告主信息表")
@RestController
@RequestMapping("/ctop/advertiser")
public class AdvertiserController {
    @Autowired
    private IAdvertiserService advertiserService;
    @Autowired
    private ISysCategoryService sysCategoryService;
    @Autowired
    private UserCompanyMapper userCompanyMapper;
    @Autowired
    private ISysRoleService sysRoleService;
    @Autowired
    private ProjectMapper projectMapper;
    @Autowired
    private ProjectMemberMapper projectMemberMapper;
    @Autowired
    private IUserAllocationService userAllocationService;


    /**
     * 查询 公司广告主
     *
     * @param userId
     * @return
     */
    @GetMapping(value = "/getAdvertiserByCompany")
    public Result<List<Advertiser>> getAdvertiserByCompany(String userId) {
        Result<List<Advertiser>> result = new Result<>();
        try {
            String roleCode = sysRoleService.getRoleCodeByUserId(userId);
            QueryWrapper<Advertiser> queryWrapper = new QueryWrapper<>();
            if (!"admin".equals(roleCode)) {
                QueryWrapper<UserCompany> userCompanyQueryWrapper = new QueryWrapper<>();
                userCompanyQueryWrapper.eq("user_id", userId);
                userCompanyQueryWrapper.orderByDesc("create_time");
                userCompanyQueryWrapper.last("limit 1");
                UserCompany userCompany = userCompanyMapper.selectOne(userCompanyQueryWrapper);
                queryWrapper.eq("company_id", userCompany.getCompanyId());
            }
            List<Advertiser> list = advertiserService.list(queryWrapper);
            result.setSuccess(true);
            result.setResult(list);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
        }
        return result;
    }

    /**
     * 分页列表查询
     *
     * @param advertiser
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "广告主信息表-分页列表查询")
    @ApiOperation(value = "广告主信息表-分页列表查询", notes = "广告主信息表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<Advertiser>> queryPageList(Advertiser advertiser,
                                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                   @RequestParam(name = "pageSize", defaultValue = "1000") Integer pageSize,
                                                   HttpServletRequest req) {

        Result<IPage<Advertiser>> result = new Result<>();
        String roleCode = sysRoleService.getRoleCodeByUserId(advertiser.getUserId());
        String userId = advertiser.getUserId();
        if ("admin".equals(roleCode) || "kuaishouOperationManager".equals(roleCode) || "touTiaoOperationManager".equals(roleCode) || "sale".equals(roleCode) || "saleDirector".equals(roleCode)) {
            advertiser.setUserId(null);
        }
        QueryWrapper<Advertiser> queryWrapper = QueryGenerator.initQueryWrapper(advertiser, req.getParameterMap());
        if ("kuaishouOperationManager".equals(roleCode)) {
            QueryWrapper<UserCompany> userCompanyQueryWrapper = new QueryWrapper<>();
            userCompanyQueryWrapper.eq("user_id", userId);
            userCompanyQueryWrapper.orderByDesc("create_time");
            userCompanyQueryWrapper.last("limit 1");
            UserCompany userCompany = userCompanyMapper.selectOne(userCompanyQueryWrapper);
            if (!Check.isNull(userCompany)) {
                queryWrapper.eq("company_id", userCompany.getCompanyId());
            }
        }
        Page<Advertiser> page = new Page<>(pageNo, pageSize);
        IPage<Advertiser> pageList = advertiserService.page(page, queryWrapper);
        if (pageList != null && pageList.getRecords() != null && !pageList.getRecords().isEmpty()) {
            List<Advertiser> list = new ArrayList<>();
            for (int i = 0; i < pageList.getRecords().size(); i++) {
                Advertiser advertiserRecord = pageList.getRecords().get(i);
                SysCategory sysCategory = sysCategoryService.getById(advertiserRecord.getIndustryId());
                if (sysCategory != null) {
                    advertiserRecord.setIndustryId(sysCategory.getName());
                }
                SysUser user = sysUserService.getById(advertiserRecord.getSaleId());
                if (null != user) {
                    advertiserRecord.setSaleName(user.getRealname());
                }
                list.add(advertiserRecord);
            }
            pageList.setRecords(list);
        }
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @Autowired
    private ISysUserService sysUserService;

    /**
     * 添加
     *
     * @param advertiser
     * @return
     */
    @AutoLog(value = "广告主信息表-添加")
    @ApiOperation(value = "广告主信息表-添加", notes = "广告主信息表-添加")
    @PostMapping(value = "/add")
    public Result<Advertiser> add(@RequestBody Advertiser advertiser) {
        Result<Advertiser> result = new Result<>();
        try {
            String userId = advertiser.getUserId();
            String name = advertiser.getName().trim();
            QueryWrapper<UserCompany> userCompanyQueryWrapper = new QueryWrapper<>();
            userCompanyQueryWrapper.eq("user_id", userId);
            userCompanyQueryWrapper.orderByDesc("create_time");
            userCompanyQueryWrapper.last("limit 1");
            UserCompany userCompany = userCompanyMapper.selectOne(userCompanyQueryWrapper);
            QueryWrapper<Advertiser> advertiserQueryWrapper = new QueryWrapper<>();
            advertiserQueryWrapper.eq("name", name);
            if (!Check.isNull(userCompany)) {
                advertiserQueryWrapper.eq("company_id", userCompany.getCompanyId());
            }
            advertiserQueryWrapper.last("limit 1");
            advertiserQueryWrapper.orderByDesc("create_time");
            Advertiser checkAdvertiser = advertiserService.getOne(advertiserQueryWrapper);
            if (!Check.isNull(checkAdvertiser)) {
                throw new Exception("广告主已存在");
            }
            if (!Check.isNull(userCompany)) {
                advertiser.setCompanyId(userCompany.getCompanyId());
            }
            advertiserService.save(advertiser);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("添加失败 " + e.getMessage());
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param advertiser
     * @return
     */
    @Autowired
    private IProjectService projectService;

    @AutoLog(value = "广告主信息表-编辑")
    @ApiOperation(value = "广告主信息表-编辑", notes = "广告主信息表-编辑")
    @PutMapping(value = "/edit")
    public Result<Advertiser> edit(@RequestBody Advertiser advertiser) {
        Result<Advertiser> result = new Result<>();
        Advertiser advertiserEntity = advertiserService.getById(advertiser.getId());
        if (advertiserEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = advertiserService.updateById(advertiser);
            QueryWrapper<Project> projectQueryWrapper = new QueryWrapper<>();
            projectQueryWrapper.eq("advertiser_id", advertiser.getId());
            Project updateProject = new Project();
            updateProject.setAdvertiserName(advertiser.getName());
            projectService.update(updateProject, projectQueryWrapper);
            QueryWrapper<UserAllocation> userAllocationQueryWrapper = new QueryWrapper<>();
            userAllocationQueryWrapper.eq("advertiser_id", advertiser.getId());
            UserAllocation updateUserAllocation = new UserAllocation();
            updateUserAllocation.setAdvertiserName(advertiser.getName());
            userAllocationService.update(updateUserAllocation, userAllocationQueryWrapper);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告主信息表-通过id删除")
    @ApiOperation(value = "广告主信息表-通过id删除", notes = "广告主信息表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Advertiser> delete(@RequestParam(name = "id", required = true) String id) {
        Result<Advertiser> result = new Result<>();
        Advertiser advertiser = advertiserService.getById(id);
        if (advertiser == null) {
            result.error500("未找到对应实体");
        } else {
            QueryWrapper<Project> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("advertiser_id", id);
            List<Project> projects = projectMapper.selectList(queryWrapper);
            if (!Check.isNull(projects)) {
                for (Project project : projects) {
                    Map<String, Object> map = new HashMap<>();
                    map.put("project_id", project.getId());
                    projectMemberMapper.deleteByMap(map);
                }

            }
            projectMapper.delete(queryWrapper);
            boolean ok = advertiserService.removeById(id);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "广告主信息表-批量删除")
    @ApiOperation(value = "广告主信息表-批量删除", notes = "广告主信息表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<Advertiser> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<Advertiser> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.advertiserService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告主信息表-通过id查询")
    @ApiOperation(value = "广告主信息表-通过id查询", notes = "广告主信息表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<Advertiser> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<Advertiser> result = new Result<>();
        Advertiser advertiser = advertiserService.getById(id);
        if (advertiser == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(advertiser);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出伊对报表
     *
     * @return
     */
    @RequestMapping(value = "/ydWeekReportList")
    public Map<String, Object> ydReportList(HttpServletRequest request, HttpServletResponse response, @RequestBody JSONObject data) {
        Map<String, Object> result = new HashMap<>();
        Long projectId = data.getLong("projectId");
        String startDate = data.getString("startDate");
        String endDate = data.getString("endDate");
        List<SheetInfoVo> vos = advertiserService.getYdStatisticInfo(projectId, startDate, endDate);
        result.put("data", vos);
        ResultMapUtils.setResultMap(result, StatusCode.COMMON_SUCCESS);
        return result;
    }

    /**
     * 导出报表
     *
     * @return
     */
    @RequestMapping(value = "/exportYdDailyReport")
    @ResponseBody
    public void exportYdWeek(HttpServletRequest request, HttpServletResponse response, @RequestBody JSONObject data) throws Exception {
        Long projectId = data.getLong("projectId");
        String startDate = data.getString("startDate");
        String endDate = data.getString("endDate");
        List<SheetInfoVo> vos = advertiserService.getYdStatisticInfo(projectId, startDate, endDate);
        OutputStream outputStream = response.getOutputStream();
        HttpUtils.setResponseHeader(response, "伊对" + startDate + "至" + endDate + "周报.xls");
        ExcelWriter excelWriter = EasyExcelFactory.getWriter(outputStream);
        for (int i = 0; i < vos.size(); i++) {
            Sheet sheet = new Sheet(i + 1, 0, KuaishouXxlEntity.class);
            sheet.setSheetName(vos.get(i).getSheetName());
            excelWriter.write(vos.get(i).getDetails(), sheet);
        }
        excelWriter.finish();
        outputStream.flush();
        outputStream.close();
    }

    /**
     * 导出消消乐报表
     *
     * @return
     */
    @RequestMapping(value = "/xxlDailyReportList")
    @ResponseBody
    public Map<String, Object> xxlReportList(HttpServletRequest request, HttpServletResponse response, @RequestBody JSONObject data) {
        Map<String, Object> result = new HashMap<>();
        List<SheetInfoVo> sheets = new ArrayList<>();
        //查询总的消耗信息
        Long projectId = data.getLong("projectId");
        String systemType = data.getString("systemType");
        if (null == systemType || "".equals(systemType.trim())) {
            systemType = null;
        }
        SheetInfoVo totalSheetVo = advertiserService.getXxlStatisticInfo(projectId, systemType, null, "汇总表");
        sheets.add(totalSheetVo);
        if (null == systemType) {
            List<SheetInfoVo> vos = advertiserService.getSystemTypeXxlStatisticInfo(projectId);
            if (null != vos && !vos.isEmpty()) {
                for (SheetInfoVo vo : vos) {
                    sheets.add(vo);
                }
            }
        }
        List<UserAllocation> allocations = userAllocationService.getByProjectId(projectId, systemType);
        if (null != allocations && !allocations.isEmpty()) {
            for (UserAllocation allocation : allocations) {
                SheetInfoVo vo = advertiserService.getXxlStatisticInfo(projectId, systemType, allocation.getAccountId(), allocation.getAuthName());
                sheets.add(vo);
            }
        }
        result.put("data", sheets);
        ResultMapUtils.setResultMap(result, StatusCode.COMMON_SUCCESS);
        return result;
    }

    /**
     * 导出报表
     *
     * @return
     */
    @RequestMapping(value = "/exportXxlDailyReport")
    @ResponseBody
    public void exportXxl(HttpServletRequest request, HttpServletResponse response, Long projectId, String systemType) throws IOException {
        List<SheetInfoVo> sheets = new ArrayList<>();
        if (null == systemType || "".equals(systemType.trim())) {
            systemType = null;
        }
        //查询总的消耗信息
        SheetInfoVo totalSheetVo = advertiserService.getXxlStatisticInfo(projectId, systemType, null, "汇总表");
        sheets.add(totalSheetVo);
        if (null == systemType) {
            List<SheetInfoVo> vos = advertiserService.getSystemTypeXxlStatisticInfo(projectId);
            if (null != vos && !vos.isEmpty()) {
                for (SheetInfoVo vo : vos) {
                    sheets.add(vo);
                }
            }
        }
        List<UserAllocation> allocations = userAllocationService.getByProjectId(projectId, systemType);
        if (null != allocations && !allocations.isEmpty()) {
            for (UserAllocation allocation : allocations) {
                SheetInfoVo vo = advertiserService.getXxlStatisticInfo(projectId, systemType, allocation.getAccountId(), allocation.getAuthName());
                sheets.add(vo);
            }
        }

        OutputStream outputStream = response.getOutputStream();
        String date = DateUtils.formatDate(new Date());
        HttpUtils.setResponseHeader(response, "爱上消消消日报" + date + ".xls");
        ExcelWriter excelWriter = EasyExcelFactory.getWriter(outputStream);
        for (int i = 0; i < sheets.size(); i++) {
            Sheet sheet = new Sheet(i + 1, 0, KuaishouXxlEntity.class);
            sheet.setSheetName(sheets.get(i).getSheetName());
            excelWriter.write(sheets.get(i).getDetails(), sheet);
        }
        excelWriter.finish();
        outputStream.flush();
        outputStream.close();
    }
}
