package org.jeecg.modules.ctop.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.ctop.entity.AgentInfo;
import org.jeecg.modules.ctop.service.IAgentInfoService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 代理商信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-02-18
 */
@Slf4j
@Api(tags = "代理商信息")
@RestController
@RequestMapping("/ctop/agentInfo")
public class AgentInfoController {
    @Autowired
    private IAgentInfoService agentInfoService;

    /**
     * 分页列表查询
     *
     * @param agentInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "代理商信息-分页列表查询")
    @ApiOperation(value = "代理商信息-分页列表查询", notes = "代理商信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<AgentInfo>> queryPageList(AgentInfo agentInfo,
                                                  @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                  HttpServletRequest req) {
        Result<IPage<AgentInfo>> result = new Result<>();
        agentInfo.setEnabled(1);
        QueryWrapper<AgentInfo> queryWrapper = QueryGenerator.initQueryWrapper(agentInfo, req.getParameterMap());
        Page<AgentInfo> page = new Page<>(pageNo, pageSize);
        IPage<AgentInfo> pageList = agentInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param agentInfo
     * @return
     */
    @AutoLog(value = "代理商信息-添加")
    @ApiOperation(value = "代理商信息-添加", notes = "代理商信息-添加")
    @PostMapping(value = "/add")
    public Result<AgentInfo> add(@RequestBody AgentInfo agentInfo) {
        Result<AgentInfo> result = new Result<>();
        try {
            agentInfo.setEnabled(1);
            agentInfo.setEnabled(1);
            agentInfoService.save(agentInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param agentInfo
     * @return
     */
    @AutoLog(value = "代理商信息-编辑")
    @ApiOperation(value = "代理商信息-编辑", notes = "代理商信息-编辑")
    @PutMapping(value = "/edit")
    public Result<AgentInfo> edit(@RequestBody AgentInfo agentInfo) {
        Result<AgentInfo> result = new Result<>();
        AgentInfo agentInfoEntity = agentInfoService.getById(agentInfo.getId());
        if (agentInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = agentInfoService.updateById(agentInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "代理商信息-通过id删除")
    @ApiOperation(value = "代理商信息-通过id删除", notes = "代理商信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            agentInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "代理商信息-批量删除")
    @ApiOperation(value = "代理商信息-批量删除", notes = "代理商信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<AgentInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<AgentInfo> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.agentInfoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "代理商信息-通过id查询")
    @ApiOperation(value = "代理商信息-通过id查询", notes = "代理商信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<AgentInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<AgentInfo> result = new Result<>();
        AgentInfo agentInfo = agentInfoService.getById(id);
        if (agentInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(agentInfo);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<AgentInfo> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, StandardCharsets.UTF_8.toString());
                AgentInfo agentInfo = JSON.parseObject(deString, AgentInfo.class);
                queryWrapper = QueryGenerator.initQueryWrapper(agentInfo, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<AgentInfo> pageList = agentInfoService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "代理商信息列表");
        mv.addObject(NormalExcelConstants.CLASS, AgentInfo.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("代理商信息列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @PostMapping("/importExcel")
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<AgentInfo> listAgentInfos = ExcelImportUtil.importExcel(file.getInputStream(), AgentInfo.class, params);
                agentInfoService.saveBatch(listAgentInfos);
                return Result.ok("文件导入成功！数据行数:" + listAgentInfos.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }
}
