package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.BindAccountAuth;
import cn.com.ctop.common.module.service.IBindAccountAuthService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.StringUtils;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.util.Arrays;

/**
 * @Description: 账号绑定-授权
 * @Author: jeecg-boot
 * @Date: 2019-07-31
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "账号绑定-授权")
@RestController
@RequestMapping("/ctop/bindAccountAuth")
public class BindAccountAuthController {
    @Autowired
    private IBindAccountAuthService bindAccountAuthService;

    /**
     * 参数校验
     *
     * @param json
     * @return
     */
    @PostMapping(value = "/authorization")
    public Result<IPage<BindAccountAuth>> authorization(@RequestBody JSONObject json) {
        log.info("授权入参:{}", json);
        Result<IPage<BindAccountAuth>> result = new Result<>();
        String codeUrl = null;
        try {
            String advertiserId = json.getString("advertiserId");
            if (Check.isNull(advertiserId)) {
                result.setSuccess(false);
                return result;
            }

            String authorizationType = json.getString("authorizationType");
            // 0 三方代理商 1 本代理商
            Integer agentType = json.getInteger("agentType");
            if ("kuaishou".equals(authorizationType)) {
                codeUrl = bindAccountAuthService.getKuaishouCodeUrl(json.toJSONString(), agentType);
            }
            if ("bytedance".equals(authorizationType)) {
                codeUrl = bindAccountAuthService.getByteDanceCodeUrl(json.toJSONString());
            }
            result.setMessage(codeUrl);
            result.setSuccess(true);
            return result;
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        result.setMessage(codeUrl);
        result.setSuccess(false);
        return result;

    }


    /**
     * 分页列表查询
     *
     * @param bindAccountAuth
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "账号绑定-授权-分页列表查询")
    @ApiOperation(value = "账号绑定-授权-分页列表查询", notes = "账号绑定-授权-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<BindAccountAuth>> queryPageList(BindAccountAuth bindAccountAuth,
                                                        @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                        HttpServletRequest req) {
        Result<IPage<BindAccountAuth>> result = new Result<>();
        QueryWrapper<BindAccountAuth> queryWrapper = QueryGenerator.initQueryWrapper(bindAccountAuth, req.getParameterMap());
        Page<BindAccountAuth> page = new Page<>(pageNo, pageSize);
        IPage<BindAccountAuth> pageList = bindAccountAuthService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param bindAccountAuth
     * @return
     */
    @AutoLog(value = "账号绑定-授权-添加")
    @ApiOperation(value = "账号绑定-授权-添加", notes = "账号绑定-授权-添加")
    @PostMapping(value = "/add")
    public Result<BindAccountAuth> add(@RequestBody BindAccountAuth bindAccountAuth) {
        Result<BindAccountAuth> result = new Result<>();
        try {
            bindAccountAuthService.save(bindAccountAuth);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param bindAccountAuth
     * @return
     */
    @AutoLog(value = "账号绑定-授权-编辑")
    @ApiOperation(value = "账号绑定-授权-编辑", notes = "账号绑定-授权-编辑")
    @PutMapping(value = "/edit")
    public Result<BindAccountAuth> edit(@RequestBody BindAccountAuth bindAccountAuth) {
        Result<BindAccountAuth> result = new Result<>();
        BindAccountAuth bindAccountAuthEntity = bindAccountAuthService.getById(bindAccountAuth.getId());
        if (bindAccountAuthEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = bindAccountAuthService.updateById(bindAccountAuth);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "账号绑定-授权-通过id删除")
    @ApiOperation(value = "账号绑定-授权-通过id删除", notes = "账号绑定-授权-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            bindAccountAuthService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "账号绑定-授权-批量删除")
    @ApiOperation(value = "账号绑定-授权-批量删除", notes = "账号绑定-授权-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<BindAccountAuth> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<BindAccountAuth> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.bindAccountAuthService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "账号绑定-授权-通过id查询")
    @ApiOperation(value = "账号绑定-授权-通过id查询", notes = "账号绑定-授权-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<BindAccountAuth> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<BindAccountAuth> result = new Result<>();
        BindAccountAuth bindAccountAuth = bindAccountAuthService.getById(id);
        if (bindAccountAuth == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(bindAccountAuth);
            result.setSuccess(true);
        }
        return result;
    }
}
