package org.jeecg.modules.ctop.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.ctop.entity.KuaiShouDefectMaterial;
import org.jeecg.modules.ctop.service.IKuaiShouDefectMaterialService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 快手缺失素材
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-04-15
 */
@Slf4j
@Api(tags = "快手缺失素材")
@RestController
@RequestMapping("/ctop/kuaiShouDefectMaterial")
public class KuaiShouDefectMaterialController {
    @Autowired
    private IKuaiShouDefectMaterialService kuaiShouDefectMaterialService;

    /**
     * 分页列表查询
     *
     * @param kuaiShouDefectMaterial
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "快手缺失素材-分页列表查询")
    @ApiOperation(value = "快手缺失素材-分页列表查询", notes = "快手缺失素材-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaiShouDefectMaterial>> queryPageList(KuaiShouDefectMaterial kuaiShouDefectMaterial,
                                                               @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                               @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                               HttpServletRequest req) {
        Result<IPage<KuaiShouDefectMaterial>> result = new Result<>();
        QueryWrapper<KuaiShouDefectMaterial> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouDefectMaterial, req.getParameterMap());
        Page<KuaiShouDefectMaterial> page = new Page<>(pageNo, pageSize);
        IPage<KuaiShouDefectMaterial> pageList = kuaiShouDefectMaterialService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaiShouDefectMaterial
     * @return
     */
    @AutoLog(value = "快手缺失素材-添加")
    @ApiOperation(value = "快手缺失素材-添加", notes = "快手缺失素材-添加")
    @PostMapping(value = "/add")
    public Result<KuaiShouDefectMaterial> add(@RequestBody KuaiShouDefectMaterial kuaiShouDefectMaterial) {
        Result<KuaiShouDefectMaterial> result = new Result<>();
        try {
            kuaiShouDefectMaterialService.save(kuaiShouDefectMaterial);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaiShouDefectMaterial
     * @return
     */
    @AutoLog(value = "快手缺失素材-编辑")
    @ApiOperation(value = "快手缺失素材-编辑", notes = "快手缺失素材-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaiShouDefectMaterial> edit(@RequestBody KuaiShouDefectMaterial kuaiShouDefectMaterial) {
        Result<KuaiShouDefectMaterial> result = new Result<>();
        KuaiShouDefectMaterial kuaiShouDefectMaterialEntity = kuaiShouDefectMaterialService.getById(kuaiShouDefectMaterial.getId());
        if (kuaiShouDefectMaterialEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaiShouDefectMaterialService.updateById(kuaiShouDefectMaterial);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手缺失素材-通过id删除")
    @ApiOperation(value = "快手缺失素材-通过id删除", notes = "快手缺失素材-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaiShouDefectMaterialService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "快手缺失素材-批量删除")
    @ApiOperation(value = "快手缺失素材-批量删除", notes = "快手缺失素材-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouDefectMaterial> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaiShouDefectMaterial> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouDefectMaterialService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手缺失素材-通过id查询")
    @ApiOperation(value = "快手缺失素材-通过id查询", notes = "快手缺失素材-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouDefectMaterial> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouDefectMaterial> result = new Result<>();
        KuaiShouDefectMaterial kuaiShouDefectMaterial = kuaiShouDefectMaterialService.getById(id);
        if (kuaiShouDefectMaterial == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouDefectMaterial);
            result.setSuccess(true);
        }
        return result;
    }
}
