package org.jeecg.modules.ctop.controller;

import cn.com.ctop.crawler.modules.kuaishoucce.entity.KuaishouHotPhotoInfo;
import cn.com.ctop.crawler.modules.kuaishoucce.service.IKuaishouHotPhotoInfoService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 快手热门作品信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-03-03
 */
@Slf4j
@Api(tags = "快手热门作品信息")
@RestController
@RequestMapping("/ctop/kuaishouHotPhotoInfo")
public class KuaishouHotPhotoInfoController {
    @Autowired
    private IKuaishouHotPhotoInfoService kuaishouHotPhotoInfoService;

    /**
     * 分页列表查询
     *
     * @param kuaishouHotPhotoInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "快手热门作品信息-分页列表查询")
    @ApiOperation(value = "快手热门作品信息-分页列表查询", notes = "快手热门作品信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouHotPhotoInfo>> queryPageList(KuaishouHotPhotoInfo kuaishouHotPhotoInfo,
                                                             @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                             @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                             @RequestParam(name = "sortType", defaultValue = "1") Integer sortType,
                                                             HttpServletRequest req) {
        Result<IPage<KuaishouHotPhotoInfo>> result = new Result<>();
        QueryWrapper<KuaishouHotPhotoInfo> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouHotPhotoInfo, req.getParameterMap());
        if (sortType == 1) {
            queryWrapper.orderByDesc("create_time");
        } else if (sortType == 2) {
            queryWrapper.orderByDesc("playing_time");
        } else if (sortType == 3) {
            queryWrapper.orderByDesc("like_time");
        } else {
            queryWrapper.orderByDesc("comment_time");
        }
        Page<KuaishouHotPhotoInfo> page = new Page<>(pageNo, pageSize);
        IPage<KuaishouHotPhotoInfo> pageList = kuaishouHotPhotoInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaishouHotPhotoInfo
     * @return
     */
    @AutoLog(value = "快手热门作品信息-添加")
    @ApiOperation(value = "快手热门作品信息-添加", notes = "快手热门作品信息-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouHotPhotoInfo> add(@RequestBody KuaishouHotPhotoInfo kuaishouHotPhotoInfo) {
        Result<KuaishouHotPhotoInfo> result = new Result<KuaishouHotPhotoInfo>();
        try {
            kuaishouHotPhotoInfoService.save(kuaishouHotPhotoInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaishouHotPhotoInfo
     * @return
     */
    @AutoLog(value = "快手热门作品信息-编辑")
    @ApiOperation(value = "快手热门作品信息-编辑", notes = "快手热门作品信息-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaishouHotPhotoInfo> edit(@RequestBody KuaishouHotPhotoInfo kuaishouHotPhotoInfo) {
        Result<KuaishouHotPhotoInfo> result = new Result<KuaishouHotPhotoInfo>();
        KuaishouHotPhotoInfo kuaishouHotPhotoInfoEntity = kuaishouHotPhotoInfoService.getById(kuaishouHotPhotoInfo.getId());
        if (kuaishouHotPhotoInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouHotPhotoInfoService.updateById(kuaishouHotPhotoInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手热门作品信息-通过id删除")
    @ApiOperation(value = "快手热门作品信息-通过id删除", notes = "快手热门作品信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaishouHotPhotoInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "快手热门作品信息-批量删除")
    @ApiOperation(value = "快手热门作品信息-批量删除", notes = "快手热门作品信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouHotPhotoInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaishouHotPhotoInfo> result = new Result<KuaishouHotPhotoInfo>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouHotPhotoInfoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手热门作品信息-通过id查询")
    @ApiOperation(value = "快手热门作品信息-通过id查询", notes = "快手热门作品信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouHotPhotoInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouHotPhotoInfo> result = new Result<KuaishouHotPhotoInfo>();
        KuaishouHotPhotoInfo kuaishouHotPhotoInfo = kuaishouHotPhotoInfoService.getById(id);
        if (kuaishouHotPhotoInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouHotPhotoInfo);
            result.setSuccess(true);
        }
        return result;
    }
}
