package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.MaterialCutFrame;
import cn.com.ctop.common.module.service.IMaterialCutFrameService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;


/**
 * 截屏
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-12-25
 */
@Slf4j
@Api(tags = "截屏")
@RestController
@RequestMapping("/cutFrame/material")
public class MaterialCutFrameController {
    @Autowired
    private IMaterialCutFrameService materialCutFrameService;

    /**
     * 分页列表查询
     *
     * @param materialCutFrame
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "截屏-分页列表查询")
    @ApiOperation(value = "截屏-分页列表查询", notes = "截屏-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<MaterialCutFrame>> queryPageList(MaterialCutFrame materialCutFrame,
                                                         @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                         @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                         HttpServletRequest req) {
        Result<IPage<MaterialCutFrame>> result = new Result<IPage<MaterialCutFrame>>();
        QueryWrapper<MaterialCutFrame> queryWrapper = QueryGenerator.initQueryWrapper(materialCutFrame, req.getParameterMap());
        Page<MaterialCutFrame> page = new Page<MaterialCutFrame>(pageNo, pageSize);
        IPage<MaterialCutFrame> pageList = materialCutFrameService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param materialCutFrame
     * @return
     */
    @AutoLog(value = "截屏-添加")
    @ApiOperation(value = "截屏-添加", notes = "截屏-添加")
    @PostMapping(value = "/add")
    public Result<MaterialCutFrame> add(@RequestBody MaterialCutFrame materialCutFrame) {
        Result<MaterialCutFrame> result = new Result<MaterialCutFrame>();
        try {
            materialCutFrameService.save(materialCutFrame);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param materialCutFrame
     * @return
     */
    @AutoLog(value = "截屏-编辑")
    @ApiOperation(value = "截屏-编辑", notes = "截屏-编辑")
    @PutMapping(value = "/edit")
    public Result<MaterialCutFrame> edit(@RequestBody MaterialCutFrame materialCutFrame) {
        Result<MaterialCutFrame> result = new Result<MaterialCutFrame>();
        MaterialCutFrame materialCutFrameEntity = materialCutFrameService.getById(materialCutFrame.getId());
        if (materialCutFrameEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = materialCutFrameService.updateById(materialCutFrame);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "截屏-通过id删除")
    @ApiOperation(value = "截屏-通过id删除", notes = "截屏-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            materialCutFrameService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "截屏-批量删除")
    @ApiOperation(value = "截屏-批量删除", notes = "截屏-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<MaterialCutFrame> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<MaterialCutFrame> result = new Result<MaterialCutFrame>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.materialCutFrameService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "截屏-通过id查询")
    @ApiOperation(value = "截屏-通过id查询", notes = "截屏-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<MaterialCutFrame> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<MaterialCutFrame> result = new Result<MaterialCutFrame>();
        MaterialCutFrame materialCutFrame = materialCutFrameService.getById(id);
        if (materialCutFrame == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(materialCutFrame);
            result.setSuccess(true);
        }
        return result;
    }
}
