package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.MaterialImageInfo;
import cn.com.ctop.common.module.entity.MaterialInfo;
import cn.com.ctop.common.module.mapper.MaterialInfoMapper;
import cn.com.ctop.common.module.service.IMaterialImageInfoService;
import cn.com.ctop.common.module.service.IMaterialInfoService;
import cn.com.ctop.common.module.utils.Check;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.DateUtils;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.ctop.service.IProjectMemberService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 素材
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-03-16
 */
@Slf4j
@Api(tags = "素材")
@RestController
@RequestMapping("/ctop/MaterialImageInfo")
public class MaterialImageInfoController {
    @Autowired
    private IMaterialImageInfoService MaterialImageInfoService;
    @Autowired
    private IMaterialInfoService materialInfoService;
    @Autowired
    private MaterialInfoMapper materialInfoMapper;
    @Autowired
    private IProjectMemberService memberService;


    @ApiOperation(value = "截屏-分页列表查询", notes = "截屏-分页列表查询")
    @GetMapping(value = "/pageList")
    public Result<IPage<MaterialImageInfo>> pageList(MaterialImageInfo materialImageInfo,
                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                     HttpServletRequest req) {
        Result<IPage<MaterialImageInfo>> result = new Result<IPage<MaterialImageInfo>>();


        QueryWrapper<MaterialImageInfo> queryWrapper = new QueryWrapper<>();
        Object createTime = materialImageInfo.getCreateTime();
        materialImageInfo.setCreateTime(null);
        if (!Check.isNull(materialImageInfo.getUserId())) {
            String roleCode = materialInfoService.getRoleCodeByUserId(materialImageInfo.getUserId());
            if ("designTeamLeader".equals(roleCode) || "planeLeader".equals(roleCode) || "admin".equals(roleCode) || "operator".equals(roleCode) || "kuaishouOperationManager".equals(roleCode) || "operationAssistant".equals(roleCode) || "touTiaoOperationManager".equals(roleCode) || "sale".equals(roleCode) || "saleDirector".equals(roleCode)) {
                materialImageInfo.setUserId(null);
                queryWrapper = QueryGenerator.initQueryWrapper(materialImageInfo, req.getParameterMap());
            } else if ("plane".equals(roleCode) || "plan".equals(roleCode) || "shot".equals(roleCode)) {
                queryWrapper = QueryGenerator.initQueryWrapper(materialImageInfo, req.getParameterMap());
            }
            if (!Check.isNull(createTime)) {
                try {
                    String dateString = DateUtils.getDateString(String.valueOf(createTime));
                    Map<String, Object> map = DateUtils.getStartEndTime(dateString);
                    queryWrapper.ge("create_time", map.get("start"));
                    queryWrapper.lt("create_time", map.get("end"));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
        }
        Page<MaterialImageInfo> page = new Page<>(pageNo, pageSize);
        IPage<MaterialImageInfo> pageList = MaterialImageInfoService.page(page, queryWrapper);


        result.setSuccess(true);
        result.setResult(pageList);
        return result;

    }


    /**
     * 分页列表查询
     * * @return
     */
    @AutoLog(value = "素材-分页列表查询")
    @ApiOperation(value = "素材-分页列表查询", notes = "素材-分页列表查询")
    @GetMapping(value = "/list")
    public Result<List<MaterialImageInfo>> queryPageList(String videoId) {
        Result<List<MaterialImageInfo>> result = new Result<>();

        List<MaterialImageInfo> list = new ArrayList<>();
        if (!Check.isNull(videoId)) {
            QueryWrapper<MaterialImageInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("video_id", videoId);
            list = MaterialImageInfoService.list(queryWrapper);
        }

        result.setSuccess(true);
        result.setResult(list);
        return result;
    }


    @GetMapping(value = "/getImageList")
    public Result<List<MaterialImageInfo>> getImageList(MaterialImageInfo MaterialImageInfo,
                                                        HttpServletRequest req) {
        Result<List<MaterialImageInfo>> result = new Result<>();
        QueryWrapper<MaterialImageInfo> queryWrapper = QueryGenerator.initQueryWrapper(MaterialImageInfo, req.getParameterMap());
        List<MaterialImageInfo> list = MaterialImageInfoService.list(queryWrapper);
        result.setSuccess(true);
        result.setResult(list);
        return result;
    }


    @GetMapping(value = "/getDetail")
    public Result<MaterialImageInfo> getDetail(String id) {
        Result<MaterialImageInfo> result = new Result<>();
        MaterialImageInfo materialImageInfo = MaterialImageInfoService.getById(id);
        if (!Check.isNull(materialImageInfo)) {
            QueryWrapper<MaterialInfo> materialInfoQueryWrapper = new QueryWrapper<>();
            materialInfoQueryWrapper.eq("code", materialImageInfo.getVideoId());
            materialInfoQueryWrapper.orderByDesc("create_time");
            materialInfoQueryWrapper.last("limit 1");
            MaterialInfo materialInfo = materialInfoService.getOne(materialInfoQueryWrapper);
            if (!Check.isNull(materialInfo)) {
                materialImageInfo.setVideoUrl(materialInfo.getUrl());

            }
            if (!Check.isNull(materialImageInfo.getAuditorId())) {
                String auditorName = materialInfoMapper.selectUserNameById(materialImageInfo.getAuditorId());
                materialImageInfo.setAuditorName(auditorName);

            }

            if (!Check.isNull(materialImageInfo.getUserId())) {
                String userName = materialInfoMapper.selectUserNameById(materialImageInfo.getUserId());
                materialImageInfo.setUserName(userName);
            }
        }

        result.setSuccess(true);
        result.setResult(materialImageInfo);
        return result;
    }


    /**
     * 添加
     *
     * @param MaterialImageInfo
     * @return
     */
    @AutoLog(value = "素材-添加")
    @ApiOperation(value = "素材-添加", notes = "素材-添加")
    @PostMapping(value = "/add")
    public Result<MaterialImageInfo> add(@RequestBody MaterialImageInfo MaterialImageInfo) {
        Result<MaterialImageInfo> result = new Result<MaterialImageInfo>();
        try {
            MaterialImageInfoService.save(MaterialImageInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param
     * @return
     */
    @AutoLog(value = "素材-编辑")

    @ApiOperation(value = "素材-编辑", notes = "素材-编辑")
    @PutMapping(value = "/edit")
    public Result<MaterialImageInfo> edit(@RequestBody MaterialImageInfo MaterialImageInfo) {
        Result<MaterialImageInfo> result = new Result<MaterialImageInfo>();
        MaterialImageInfo MaterialImageInfoEntity = MaterialImageInfoService.getById(MaterialImageInfo.getId());
        if (MaterialImageInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            String url = (String) MaterialImageInfo.getUrl();
            String urlStr = "https:" + url;
            MaterialImageInfo.setUrl(urlStr);
            boolean ok = MaterialImageInfoService.updateById(MaterialImageInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }


    @AutoLog(value = "素材-编辑")
    @ApiOperation(value = "素材-编辑", notes = "素材-编辑")
    @PutMapping(value = "/batchEdit")
    public Result<MaterialImageInfo> batchEdit(@RequestBody JSONArray updateArr) {
        Result<MaterialImageInfo> result = new Result<MaterialImageInfo>();
        try {
            if (!Check.isNull(updateArr)) {
                LoginUser user = (LoginUser) SecurityUtils.getSubject()
                        .getPrincipal();
                for (int i = 0; i < updateArr.size(); i++) {
                    JSONObject updateJson = updateArr.getJSONObject(i);
                    if (!Check.isNull(updateJson)) {
                        MaterialImageInfo materialImageInfo = new MaterialImageInfo();
                        Long id = updateJson.getLong("id");
                        materialImageInfo.setId(id);
                        Integer status = updateJson.getInteger("status");
                        materialImageInfo.setStatus(status);
                        if (status == 2) {
                            materialImageInfo.setRefuseReason(updateJson.getString("refuseReason"));
                        }
                        materialImageInfo.setAuditorId(user.getId());
                        MaterialImageInfoService.updateById(materialImageInfo);
                    }
                }
            }
            result.success("修改成功!");
            result.setSuccess(true);
        } catch (Exception e) {
            result.setSuccess(false);
            e.printStackTrace();
        }

        return result;
    }


    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "素材-通过id删除")
    @ApiOperation(value = "素材-通过id删除", notes = "素材-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            MaterialImageInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "素材-批量删除")
    @ApiOperation(value = "素材-批量删除", notes = "素材-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<MaterialImageInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<MaterialImageInfo> result = new Result<MaterialImageInfo>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.MaterialImageInfoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "素材-通过id查询")
    @ApiOperation(value = "素材-通过id查询", notes = "素材-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<MaterialImageInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<MaterialImageInfo> result = new Result<MaterialImageInfo>();
        MaterialImageInfo MaterialImageInfo = MaterialImageInfoService.getById(id);
        if (MaterialImageInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(MaterialImageInfo);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<MaterialImageInfo> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                MaterialImageInfo MaterialImageInfo = JSON.parseObject(deString, MaterialImageInfo.class);
                queryWrapper = QueryGenerator.initQueryWrapper(MaterialImageInfo, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<MaterialImageInfo> pageList = MaterialImageInfoService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "素材列表");
        mv.addObject(NormalExcelConstants.CLASS, MaterialImageInfo.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("素材列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<MaterialImageInfo> listMaterialImageInfos = ExcelImportUtil.importExcel(file.getInputStream(), MaterialImageInfo.class, params);
                MaterialImageInfoService.saveBatch(listMaterialImageInfos);
                return Result.ok("文件导入成功！数据行数:" + listMaterialImageInfos.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }
}
