package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.service.ISysRoleService;
import cn.com.ctop.common.module.utils.Check;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.ctop.entity.Project;
import org.jeecg.modules.ctop.entity.ProjectMember;
import org.jeecg.modules.ctop.mapper.ProjectMemberMapper;
import org.jeecg.modules.ctop.service.IProjectMemberService;
import org.jeecg.modules.ctop.service.IProjectService;
import org.jeecg.modules.system.entity.SysUser;
import org.jeecg.modules.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * 项目成员
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-11-19
 */
@Slf4j
@Api(tags = "项目成员")
@RestController
@RequestMapping("/ctop/projectMember")
public class ProjectMemberController {
    @Autowired
    private IProjectMemberService projectMemberService;
    @Autowired
    private ProjectMemberMapper projectMemberMapper;
    @Autowired
    private IProjectService projectService;
    @Autowired
    private ISysRoleService sysRoleService;
    @Autowired
    private ISysUserService sysUserService;


    /**
     * 查看项目成员
     *
     * @param projectId
     * @return
     */
    @AutoLog(value = "项目成员-分页列表查询")
    @ApiOperation(value = "项目成员-分页列表查询", notes = "项目成员-分页列表查询")
    @GetMapping(value = "/getMemberList")
    public Map<String, Object> getMemberList(String projectId) {
        Map<String, Object> map = new HashMap<>();
        try {
            QueryWrapper<ProjectMember> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("project_id", projectId);
            queryWrapper.orderByDesc("create_time");
            List<ProjectMember> projectMemberList = projectMemberMapper.selectList(queryWrapper);
            map.put("code", 0);
            map.put("success", true);
            map.put("result", projectMemberList);
        } catch (Exception e) {
            map.put("success", false);
            map.put("code", -1);
        }

        return map;
    }


    /**
     * 我参与项目
     *
     * @param userId
     * @return
     */
    @AutoLog(value = "项目成员-分页列表查询")
    @ApiOperation(value = "项目成员-分页列表查询", notes = "项目成员-分页列表查询")
    @GetMapping(value = "/participateList")
    public Map<String, Object> participateList(String userId) {
        Map<String, Object> map = new HashMap<>();
        try {
            QueryWrapper<ProjectMember> queryWrapper = new QueryWrapper<>();
            String roleCode = sysRoleService.getRoleCodeByUserId(userId);
            if (!"admin".equals(roleCode) && !"sale".equals(roleCode) && !"saleDirector".equals(roleCode)) {
                queryWrapper.eq("user_id", userId);
            }
            queryWrapper.groupBy("project_id");
            queryWrapper.orderByDesc("create_time");
            List<ProjectMember> projectMembers = projectMemberMapper.selectList(queryWrapper);
            List<ProjectMember> projectMemberList = new ArrayList<>();
            if (!Check.isNull(projectMembers)) {
                for (ProjectMember projectMember : projectMembers) {
                    if (Check.isNull(projectMember)) {
                        continue;
                    }
                    Project project = projectService.getById(projectMember.getProjectId());
                    if (!Check.isNull(project)) {
                        projectMember.setResponsibleName(project.getResponsibleName());
                        projectMember.setDesignResponsibleName(project.getDesignResponsibleName());
                        projectMember.setAdvertiserName(project.getAdvertiserName());
                        projectMember.setAdvertiserId(project.getAdvertiserId());
                        projectMember.setMediaId(project.getMediaId());
                        projectMember.setMaxBid(project.getMaxBid());
                        projectMemberList.add(projectMember);
                    }

                }
            }
            map.put("code", 0);
            map.put("success", true);
            map.put("result", projectMemberList);
        } catch (Exception e) {
            e.printStackTrace();
            map.put("success", false);
            map.put("code", -1);
        }

        return map;
    }


    /**
     * 分页列表查询
     *
     * @param projectMember
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "项目成员-分页列表查询")
    @ApiOperation(value = "项目成员-分页列表查询", notes = "项目成员-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ProjectMember>> queryPageList(ProjectMember projectMember,
                                                      @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                      @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                      HttpServletRequest req) {
        Result<IPage<ProjectMember>> result = new Result<>();
        QueryWrapper<ProjectMember> queryWrapper = QueryGenerator.initQueryWrapper(projectMember, req.getParameterMap());
        Page<ProjectMember> page = new Page<>(pageNo, pageSize);
        IPage<ProjectMember> pageList = projectMemberService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }


    /**
     * 批量添加
     *
     * @param jsonObject
     * @return
     */


    @AutoLog(value = "项目成员-添加")
    @ApiOperation(value = "项目成员-添加", notes = "项目成员-添加")
    @PostMapping(value = "/batchAdd")
    public Result<ProjectMember> batchAdd(@RequestBody JSONObject jsonObject) {
        Result<ProjectMember> result = new Result<>();
        try {
            if (Check.isNull(jsonObject)) {
                throw new Exception("入参不能为空");
            }
            Long projectId = jsonObject.getLong("projectId");
            String projectName = jsonObject.getString("projectName");
            JSONArray memberArr = jsonObject.getJSONArray("memberArr");
            if (Check.isNull(memberArr)) {
                throw new Exception("请选择项目成员");
            }
            for (int i = 0; i < memberArr.size(); i++) {
                String userId = memberArr.get(i).toString();
                if (!Check.isNull(userId)) {
                    QueryWrapper<ProjectMember> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("user_id", userId);
                    queryWrapper.eq("project_id", projectId);
                    ProjectMember selectMember = (ProjectMember) projectMemberService.getOne(queryWrapper);
                    if (!Check.isNull(selectMember)) {
                        continue;
                    }
                    ProjectMember member = new ProjectMember();
                    String roleCode = sysRoleService.getRoleCodeByUserId(userId);
                    if (!Check.isNull(roleCode)) {
                        member.setRoleCode(roleCode);
                    }
                    member.setProjectId(projectId);
                    member.setProjectName(projectName);
                    member.setUserId(userId);
                    SysUser sysUser = sysUserService.getById(userId);
                    if (!Check.isNull(sysUser)) {
                        member.setUserName(sysUser.getRealname());
                    }

                    projectMemberService.save(member);
                }
            }
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }


    /**
     * 添加
     *
     * @param projectMember
     * @return
     */
    @AutoLog(value = "项目成员-添加")
    @ApiOperation(value = "项目成员-添加", notes = "项目成员-添加")
    @PostMapping(value = "/add")
    public Result<ProjectMember> add(@RequestBody ProjectMember projectMember) {
        Result<ProjectMember> result = new Result<>();
        try {
            projectMemberService.save(projectMember);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param projectMember
     * @return
     */
    @AutoLog(value = "项目成员-编辑")
    @ApiOperation(value = "项目成员-编辑", notes = "项目成员-编辑")
    @PutMapping(value = "/edit")
    public Result<ProjectMember> edit(@RequestBody ProjectMember projectMember) {
        Result<ProjectMember> result = new Result<>();
        ProjectMember projectMemberEntity = projectMemberService.getById(projectMember.getId());
        if (projectMemberEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = projectMemberService.updateById(projectMember);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "项目成员-通过id删除")
    @ApiOperation(value = "项目成员-通过id删除", notes = "项目成员-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            projectMemberService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "项目成员-批量删除")
    @ApiOperation(value = "项目成员-批量删除", notes = "项目成员-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ProjectMember> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ProjectMember> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.projectMemberService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "项目成员-通过id查询")
    @ApiOperation(value = "项目成员-通过id查询", notes = "项目成员-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ProjectMember> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ProjectMember> result = new Result<>();
        ProjectMember projectMember = projectMemberService.getById(id);
        if (projectMember == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(projectMember);
            result.setSuccess(true);
        }
        return result;
    }
}
