package org.jeecg.modules.ctop.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.ctop.entity.ScriptTagInfo;
import org.jeecg.modules.ctop.service.IScriptTagInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 脚本库标签信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-02-27
 */
@Slf4j
@Api(tags = "脚本库标签信息")
@RestController
@RequestMapping("/ctop/scriptTagInfo")
public class ScriptTagInfoController {
    @Autowired
    private IScriptTagInfoService scriptTagInfoService;

    /**
     * 分页列表查询
     *
     * @param scriptTagInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "脚本库标签信息-分页列表查询")
    @ApiOperation(value = "脚本库标签信息-分页列表查询", notes = "脚本库标签信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ScriptTagInfo>> queryPageList(ScriptTagInfo scriptTagInfo,
                                                      @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                      @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                      HttpServletRequest req) {
        Result<IPage<ScriptTagInfo>> result = new Result<>();
        QueryWrapper<ScriptTagInfo> queryWrapper = QueryGenerator.initQueryWrapper(scriptTagInfo, req.getParameterMap());
        Page<ScriptTagInfo> page = new Page<>(pageNo, pageSize);
        IPage<ScriptTagInfo> pageList = scriptTagInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param scriptTagInfo
     * @return
     */
    @AutoLog(value = "脚本库标签信息-添加")
    @ApiOperation(value = "脚本库标签信息-添加", notes = "脚本库标签信息-添加")
    @PostMapping(value = "/add")
    public Result<ScriptTagInfo> add(@RequestBody ScriptTagInfo scriptTagInfo) {
        Result<ScriptTagInfo> result = new Result<>();
        try {
            scriptTagInfoService.save(scriptTagInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param scriptTagInfo
     * @return
     */
    @AutoLog(value = "脚本库标签信息-编辑")
    @ApiOperation(value = "脚本库标签信息-编辑", notes = "脚本库标签信息-编辑")
    @PutMapping(value = "/edit")
    public Result<ScriptTagInfo> edit(@RequestBody ScriptTagInfo scriptTagInfo) {
        Result<ScriptTagInfo> result = new Result<>();
        ScriptTagInfo scriptTagInfoEntity = scriptTagInfoService.getById(scriptTagInfo.getId());
        if (scriptTagInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = scriptTagInfoService.updateById(scriptTagInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "脚本库标签信息-通过id删除")
    @ApiOperation(value = "脚本库标签信息-通过id删除", notes = "脚本库标签信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            scriptTagInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "脚本库标签信息-批量删除")
    @ApiOperation(value = "脚本库标签信息-批量删除", notes = "脚本库标签信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ScriptTagInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ScriptTagInfo> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.scriptTagInfoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "脚本库标签信息-通过id查询")
    @ApiOperation(value = "脚本库标签信息-通过id查询", notes = "脚本库标签信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ScriptTagInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ScriptTagInfo> result = new Result<>();
        ScriptTagInfo scriptTagInfo = scriptTagInfoService.getById(id);
        if (scriptTagInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(scriptTagInfo);
            result.setSuccess(true);
        }
        return result;
    }
}
