package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.UserAllocation;
import cn.com.ctop.common.module.mapper.UserAllocationMapper;
import cn.com.ctop.common.module.service.ISysRoleService;
import cn.com.ctop.common.module.service.IUserAllocationService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.ctop.entity.Project;
import org.jeecg.modules.ctop.entity.ProjectMember;
import org.jeecg.modules.ctop.mapper.ProjectMemberMapper;
import org.jeecg.modules.ctop.service.IProjectMemberService;
import org.jeecg.modules.ctop.service.IProjectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.List;

/**
 * 用户分配
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-08-08
 */
@Slf4j
@Api(tags = "用户分配")
@RestController
@RequestMapping("/ctop/userAllocation")
public class UserAllocationController {
    @Autowired
    private IUserAllocationService userAllocationService;
    @Autowired
    private UserAllocationMapper userAllocationMapper;
    @Autowired
    private IProjectService projectService;
    @Autowired
    private ISysRoleService sysRoleService;
    @Autowired
    private ProjectMemberMapper memberMapper;
    @Autowired
    private IProjectMemberService projectMemberService;


    @GetMapping(value = "/accountTurnProject")
    public Result<Object> getAccountList(Long accountId, Long projectId) {
        Result<Object> result = new Result<>();
        try {
            QueryWrapper<UserAllocation> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("account_id", accountId);
            queryWrapper.eq("account_status", 0);
            queryWrapper.orderByDesc("create_time");
            queryWrapper.last("limit 1");
            UserAllocation userAllocations = userAllocationMapper.selectOne(queryWrapper);
            if (Check.isNull(userAllocations)) {
                throw new Exception("账号信息为空");
            }
            Project project = projectService.getById(projectId);
            if (Check.isNull(project)) {
                throw new Exception("项目信息为空");
            }
            QueryWrapper<UserAllocation> checkQueryWrapper = new QueryWrapper<>();
            checkQueryWrapper.eq("account_id", accountId);
            checkQueryWrapper.eq("project_id", projectId);
            checkQueryWrapper.last("limit 1");
            UserAllocation checkUserAllocation = userAllocationMapper.selectOne(checkQueryWrapper);
            if (!Check.isNull(checkUserAllocation)) {
                throw new Exception("该项目下已存在此账户");
            }
            userAllocations.setAccountStatus(1);
            userAllocations.setProjectId(projectId);
            userAllocations.setProjectName(project.getProjectName());
            userAllocations.setMediaId(project.getMediaId());
            userAllocations.setAdvertiserId(project.getAdvertiserId());
            userAllocations.setAdvertiserName(project.getAdvertiserName());
            userAllocationService.save(userAllocations);
            QueryWrapper<ProjectMember> memberQueryWrapper = new QueryWrapper<>();
            memberQueryWrapper.eq("user_id", userAllocations.getUserId());
            memberQueryWrapper.eq("project_id", projectId);
            memberQueryWrapper.last("limit 1");
            ProjectMember member = memberMapper.selectOne(memberQueryWrapper);
            if (Check.isNull(member)) {
                ProjectMember addMember = new ProjectMember();
                addMember.setProjectId(projectId);
                addMember.setProjectName(project.getProjectName());
                addMember.setUserId(userAllocations.getUserId());
                addMember.setUserName(userAllocations.getUserName());
                String roleCode = sysRoleService.getRoleCodeByUserId(userAllocations.getUserId());
                if (!Check.isNull(roleCode)) {
                    addMember.setRoleCode(roleCode);
                }
                projectMemberService.save(addMember);
            }

            result.setSuccess(true);
        } catch (Exception e) {
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }

        return result;

    }


    @GetMapping(value = "/getAccountList")
    public Result<List<UserAllocation>> getAccountList(String userId, Long projectId) {
        Result<List<UserAllocation>> result = new Result<>();
        try {

            QueryWrapper<UserAllocation> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("user_id", userId);
            queryWrapper.eq("project_id", projectId);
            Project project = projectService.getById(projectId);
            if (!Check.isNull(project)) {
                queryWrapper.eq("media_id", project.getMediaId());
            }

            List<UserAllocation> userAllocations = userAllocationMapper.selectList(queryWrapper);
            result.setSuccess(true);
            result.setResult(userAllocations);
        } catch (Exception e) {
            result.setSuccess(false);
            result.success("查询失败");
        }

        return result;

    }


    @GetMapping(value = "/getAccountsByProjectId")
    public Result<List<UserAllocation>> getAccountsByProjectId(String userId, Long projectId) {
        Result<List<UserAllocation>> result = new Result<>();
        try {
            QueryWrapper<UserAllocation> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("project_id", projectId);
            Project project = projectService.getById(projectId);
            if (!Check.isNull(project)) {
                queryWrapper.eq("media_id", project.getMediaId());
            }

            String roleCode = sysRoleService.getRoleCodeByUserId(userId);
            if (!"admin".equals(roleCode) && !"kuaishouOperationManager".equals(roleCode) && !"designTeamLeader".equals(roleCode) && !"touTiaoOperationManager".equals(roleCode)) {
                queryWrapper.eq("user_id", userId);
            }
            queryWrapper.orderByDesc("create_time");
            List<UserAllocation> userAllocations = userAllocationMapper.selectList(queryWrapper);
            result.setSuccess(true);
            result.setResult(userAllocations);
        } catch (Exception e) {
            result.setSuccess(false);
            result.success("查询失败");
        }

        return result;

    }


    /**
     * 统计报表 查看所有账号
     *
     * @param projectId
     * @return
     */
    @GetMapping(value = "/getAccountListByProjectId")
    public Result<List<UserAllocation>> getAccountListByProjectId(Long projectId) {
        Result<List<UserAllocation>> result = new Result<>();
        try {

            QueryWrapper<UserAllocation> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("project_id", projectId);
            Project project = projectService.getById(projectId);
            if (!Check.isNull(project)) {
                queryWrapper.eq("media_id", project.getMediaId());
            }
            queryWrapper.isNotNull("account_id");
            List<UserAllocation> userAllocations = userAllocationMapper.selectList(queryWrapper);
            result.setSuccess(true);
            result.setResult(userAllocations);
        } catch (Exception e) {
            result.setSuccess(false);
            result.success("查询失败");
        }

        return result;

    }


    /**
     * 分页列表查询
     *
     * @param userAllocation
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "用户分配-分页列表查询")
    @ApiOperation(value = "用户分配-分页列表查询", notes = "用户分配-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<UserAllocation>> queryPageList(UserAllocation userAllocation,
                                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                       HttpServletRequest req) {
        Result<IPage<UserAllocation>> result = new Result<>();
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        QueryWrapper<UserAllocation> queryWrapper = QueryGenerator.initQueryWrapper(userAllocation, req.getParameterMap());
        String roleCode = sysRoleService.getRoleCodeByUserId(user.getId());
        if (!"admin".equals(roleCode)) {
            queryWrapper.eq("user_id", user.getId());
            queryWrapper.orderByDesc("id");
        }
        Page<UserAllocation> page = new Page<>(pageNo, pageSize);
        IPage<UserAllocation> pageList = userAllocationService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param userAllocation
     * @return
     */
    @AutoLog(value = "用户分配-添加")
    @ApiOperation(value = "用户分配-添加", notes = "用户分配-添加")
    @PostMapping(value = "/add")
    public Result<UserAllocation> add(@RequestBody UserAllocation userAllocation) {
        Result<UserAllocation> result = new Result<>();
        try {
            userAllocationService.save(userAllocation);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param userAllocation
     * @return
     */
    @AutoLog(value = "用户分配-编辑")
    @ApiOperation(value = "用户分配-编辑", notes = "用户分配-编辑")
    @PutMapping(value = "/edit")
    public Result<UserAllocation> edit(@RequestBody UserAllocation userAllocation) {
        Result<UserAllocation> result = new Result<>();
        UserAllocation userAllocationEntity = userAllocationService.getById(userAllocation.getId());
        if (userAllocationEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = userAllocationService.updateById(userAllocation);
            QueryWrapper<ProjectMember> memberQueryWrapper = new QueryWrapper<>();
            memberQueryWrapper.eq("user_id", userAllocation.getUserId());
            memberQueryWrapper.eq("project_id", userAllocation.getProjectId());
            memberQueryWrapper.last("limit 1");
            ProjectMember member = memberMapper.selectOne(memberQueryWrapper);
            if (Check.isNull(member)) {
                ProjectMember addMember = new ProjectMember();
                addMember.setProjectId(userAllocation.getProjectId());
                addMember.setProjectName(userAllocation.getProjectName());
                addMember.setUserId(userAllocation.getUserId());
                addMember.setUserName(userAllocation.getUserName());
                String roleCode = sysRoleService.getRoleCodeByUserId(userAllocation.getUserId());
                if (!Check.isNull(roleCode)) {
                    addMember.setRoleCode(roleCode);
                }
                projectMemberService.save(addMember);
            }


            if (ok) {
                result.success("修改成功!");
            }
        }
        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "用户分配-通过id删除")
    @ApiOperation(value = "用户分配-通过id删除", notes = "用户分配-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            userAllocationService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "用户分配-批量删除")
    @ApiOperation(value = "用户分配-批量删除", notes = "用户分配-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<UserAllocation> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<UserAllocation> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.userAllocationService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "用户分配-通过id查询")
    @ApiOperation(value = "用户分配-通过id查询", notes = "用户分配-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<UserAllocation> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<UserAllocation> result = new Result<>();
        UserAllocation userAllocation = userAllocationService.getById(id);
        if (userAllocation == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(userAllocation);
            result.setSuccess(true);
        }
        return result;
    }
}
