package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.VideoWatermarkTask;
import cn.com.ctop.common.module.service.IVideoWatermarkTaskService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * 水印任务表
 * @author jeecg-boot
 * @date   2019-12-20
 * @version V1.0
 */
@Slf4j
@Api(tags="水印任务表")
@RestController
@RequestMapping("/ctop/videoWatermarkTask")
public class VideoWatermarkTaskController {
	@Autowired
	private IVideoWatermarkTaskService videoWatermarkTaskService;

	/**
	  * 分页列表查询
	 * @param videoWatermarkTask
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "水印任务表-分页列表查询")
	@ApiOperation(value="水印任务表-分页列表查询", notes="水印任务表-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<VideoWatermarkTask>> queryPageList(VideoWatermarkTask videoWatermarkTask,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
        Result<IPage<VideoWatermarkTask>> result = new Result<>();
		QueryWrapper<VideoWatermarkTask> queryWrapper = QueryGenerator.initQueryWrapper(videoWatermarkTask, req.getParameterMap());
        Page<VideoWatermarkTask> page = new Page<>(pageNo, pageSize);
		IPage<VideoWatermarkTask> pageList = videoWatermarkTaskService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
    }

	/**
	  *   添加
	 * @param videoWatermarkTask
	 * @return
	 */
	@AutoLog(value = "水印任务表-添加")
	@ApiOperation(value="水印任务表-添加", notes="水印任务表-添加")
	@PostMapping(value = "/add")
	public Result<VideoWatermarkTask> add(@RequestBody VideoWatermarkTask videoWatermarkTask) {
        Result<VideoWatermarkTask> result = new Result<>();
		try {
			videoWatermarkTaskService.save(videoWatermarkTask);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

     /**
	  *  编辑
	 * @param videoWatermarkTask
	 * @return
	 */
	@AutoLog(value = "水印任务表-编辑")
	@ApiOperation(value="水印任务表-编辑", notes="水印任务表-编辑")
	@PutMapping(value = "/edit")
	public Result<VideoWatermarkTask> edit(@RequestBody VideoWatermarkTask videoWatermarkTask) {
        Result<VideoWatermarkTask> result = new Result<>();
		VideoWatermarkTask videoWatermarkTaskEntity = videoWatermarkTaskService.getById(videoWatermarkTask.getId());
		if(videoWatermarkTaskEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = videoWatermarkTaskService.updateById(videoWatermarkTask);
			if(ok) {
				result.success("修改成功!");
			}
		}

        return result;
	}

     /**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "水印任务表-通过id删除")
	@ApiOperation(value="水印任务表-通过id删除", notes="水印任务表-通过id删除")
	@DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
		try {
			videoWatermarkTaskService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

     /**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "水印任务表-批量删除")
	@ApiOperation(value="水印任务表-批量删除", notes="水印任务表-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<VideoWatermarkTask> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
        Result<VideoWatermarkTask> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.videoWatermarkTaskService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

     /**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "水印任务表-通过id查询")
	@ApiOperation(value="水印任务表-通过id查询", notes="水印任务表-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<VideoWatermarkTask> queryById(@RequestParam(name="id",required=true) String id) {
        Result<VideoWatermarkTask> result = new Result<>();
		VideoWatermarkTask videoWatermarkTask = videoWatermarkTaskService.getById(id);
		if(videoWatermarkTask==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(videoWatermarkTask);
			result.setSuccess(true);
		}
		return result;
	}
}
