package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.VideoWatermarkTemplate;
import cn.com.ctop.common.module.service.IVideoWatermarkTemplateService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * 水印模板
 * @author jeecg-boot
 * @date   2019-12-20
 * @version V1.0
 */
@Slf4j
@Api(tags="水印模板")
@RestController
@RequestMapping("/ctop/videoWatermarkTemplate")
public class VideoWatermarkTemplateController {
	@Autowired
	private IVideoWatermarkTemplateService videoWatermarkTemplateService;

     /**
	  * 分页列表查询
	 * @param videoWatermarkTemplate
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "水印模板-分页列表查询")
	@ApiOperation(value="水印模板-分页列表查询", notes="水印模板-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<VideoWatermarkTemplate>> queryPageList(VideoWatermarkTemplate videoWatermarkTemplate,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
        Result<IPage<VideoWatermarkTemplate>> result = new Result<>();
		QueryWrapper<VideoWatermarkTemplate> queryWrapper = QueryGenerator.initQueryWrapper(videoWatermarkTemplate, req.getParameterMap());
        Page<VideoWatermarkTemplate> page = new Page<>(pageNo, pageSize);
		IPage<VideoWatermarkTemplate> pageList = videoWatermarkTemplateService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

     /**
	  *   添加
	 * @param videoWatermarkTemplate
	 * @return
	 */
	@AutoLog(value = "水印模板-添加")
	@ApiOperation(value="水印模板-添加", notes="水印模板-添加")
	@PostMapping(value = "/add")
	public Result<VideoWatermarkTemplate> add(@RequestBody VideoWatermarkTemplate videoWatermarkTemplate) {
        Result<VideoWatermarkTemplate> result = new Result<>();
		try {
			videoWatermarkTemplateService.save(videoWatermarkTemplate);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

     /**
	  *  编辑
	 * @param videoWatermarkTemplate
	 * @return
	 */
	@AutoLog(value = "水印模板-编辑")
	@ApiOperation(value="水印模板-编辑", notes="水印模板-编辑")
	@PutMapping(value = "/edit")
	public Result<VideoWatermarkTemplate> edit(@RequestBody VideoWatermarkTemplate videoWatermarkTemplate) {
        Result<VideoWatermarkTemplate> result = new Result<>();
		VideoWatermarkTemplate videoWatermarkTemplateEntity = videoWatermarkTemplateService.getById(videoWatermarkTemplate.getId());
		if(videoWatermarkTemplateEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = videoWatermarkTemplateService.updateById(videoWatermarkTemplate);
			if(ok) {
				result.success("修改成功!");
			}
		}

        return result;
	}

     /**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "水印模板-通过id删除")
	@ApiOperation(value="水印模板-通过id删除", notes="水印模板-通过id删除")
	@DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
		try {
			videoWatermarkTemplateService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

     /**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "水印模板-批量删除")
	@ApiOperation(value="水印模板-批量删除", notes="水印模板-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<VideoWatermarkTemplate> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
        Result<VideoWatermarkTemplate> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.videoWatermarkTemplateService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

     /**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "水印模板-通过id查询")
	@ApiOperation(value="水印模板-通过id查询", notes="水印模板-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<VideoWatermarkTemplate> queryById(@RequestParam(name="id",required=true) String id) {
        Result<VideoWatermarkTemplate> result = new Result<>();
		VideoWatermarkTemplate videoWatermarkTemplate = videoWatermarkTemplateService.getById(id);
		if(videoWatermarkTemplate==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(videoWatermarkTemplate);
			result.setSuccess(true);
		}
		return result;
	}
}
