package org.jeecg.modules.industry.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.industry.entity.Industry;
import org.jeecg.modules.industry.model.IndustryTree;
import org.jeecg.modules.industry.service.IIndustryService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.*;

 /**
 * @Description: 行业表
 * @Author: jeecg-boot
 * @Date:   2019-06-10
 * @Version: V1.0
 */
@Slf4j
@Api(tags="行业表")
@RestController
@RequestMapping("/industry/industry")
public class IndustryController {
	@Autowired
	private IIndustryService industryService;

     /**
	  * 分页列表查询
	 * @return
	 */
	@RequestMapping(value = "/list", method = RequestMethod.GET)
	public Result<List<IndustryTree>> list() {
		Result<List<IndustryTree>> result = new Result<List<IndustryTree>>();
		List<Industry> list = industryService.list();
		List<IndustryTree> treeList = new ArrayList<IndustryTree>();
		getTreeList(treeList,list,null);
		result.setSuccess(true);
		result.setResult(treeList);
		return result;
	}

	 /**
	  * 获取全部的行业树
	  *
	  * @return
	  */
	 @RequestMapping(value = "/queryTreeList", method = RequestMethod.GET)
	 public Result<Map<String, Object>> queryTreeList() {
		 Result<Map<String, Object>> result = new Result<>();
		 // 全部权限ids
		 List<String> ids = new ArrayList<>();
		 try {
			 List<Industry> list = industryService.list();
			 List<IndustryTree> treeList = new ArrayList<IndustryTree>();
			 for (Industry industry : list) {
				 ids.add(industry.getId());
			 }
			 getTreeList(treeList,list,null);

			 Map<String, Object> resMap = new HashMap<>();
			 // 全部树节点数据
			 resMap.put("treeList", treeList);
			 // 全部树ids
			 resMap.put("ids", ids);
			 result.setResult(resMap);
			 result.setSuccess(true);
		 } catch (Exception e) {
			 log.error(e.getMessage(), e);
		 }
		 return result;
	 }


	 private void getTreeList(List<IndustryTree> treeList, List<Industry> metaList, IndustryTree temp) {
		 for (Industry industry : metaList) {
			 String tempPid = industry.getParentId();
			 IndustryTree tree = new IndustryTree(industry);
			 if (temp == null && oConvertUtils.isEmpty(tempPid)) {
				 treeList.add(tree);
				 getTreeList(treeList,metaList,tree);
			 } else if (temp != null && tempPid != null && tempPid.equals(temp.getId())) {
			 	if(temp.getChildren() == null){
					temp.setChildren(new ArrayList<IndustryTree>());
				}
				 temp.getChildren().add(tree);
			 }

		 }
	 }

     /**
	  *   添加
	 * @param industry
	 * @return
	 */
	@AutoLog(value = "行业表-添加")
	@ApiOperation(value="行业表-添加", notes="行业表-添加")
	@PostMapping(value = "/add")
	public Result<Industry> add(@RequestBody Industry industry) {
		Result<Industry> result = new Result<>();
		try {
			industryService.save(industry);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

     /**
	  *  编辑
	 * @param industry
	 * @return
	 */
	@AutoLog(value = "行业表-编辑")
	@ApiOperation(value="行业表-编辑", notes="行业表-编辑")
	@PutMapping(value = "/edit")
	public Result<Industry> edit(@RequestBody Industry industry) {
		Result<Industry> result = new Result<>();
		Industry industryEntity = industryService.getById(industry.getId());
		if(industryEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = industryService.updateById(industry);
			if(ok) {
				result.success("修改成功!");
			}
		}
        return result;
	}

     /**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "行业表-通过id删除")
	@ApiOperation(value="行业表-通过id删除", notes="行业表-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<Industry> delete(@RequestParam(name="id",required=true) String id) {
		Result<Industry> result = new Result<>();
		Industry industry = industryService.getById(id);
		if(industry==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = industryService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
        return result;
	}

     /**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "行业表-批量删除")
	@ApiOperation(value="行业表-批量删除", notes="行业表-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<Industry> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<Industry> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.industryService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

     /**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "行业表-通过id查询")
	@ApiOperation(value="行业表-通过id查询", notes="行业表-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<Industry> queryById(@RequestParam(name="id",required=true) String id) {
		Result<Industry> result = new Result<>();
		Industry industry = industryService.getById(id);
		if(industry==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(industry);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<Industry> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              Industry industry = JSON.parseObject(deString, Industry.class);
              queryWrapper = QueryGenerator.initQueryWrapper(industry, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }
      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<Industry> pageList = industryService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "行业表列表");
      mv.addObject(NormalExcelConstants.CLASS, Industry.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("行业表列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();

		  ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<Industry> listIndustrys = ExcelImportUtil.importExcel(file.getInputStream(), Industry.class, params);
              for (Industry industryExcel : listIndustrys) {
                  industryService.save(industryExcel);
              }
              return Result.ok("文件导入成功！数据行数:" + listIndustrys.size());
          } catch (Exception e) {
              log.error(e.getMessage(),e);
              return Result.error("文件导入失败:"+e.getMessage());
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }
}
