package org.jeecg.modules.opencv.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.opencv.entity.ModelVideoInfoTag;
import org.jeecg.modules.opencv.service.IModelVideoInfoTagService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 模板视频分类信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-04-17
 */
@Slf4j
@Api(tags = "模板视频分类信息")
@RestController
@RequestMapping("/ctop/modelVideoInfoTag")
public class ModelVideoInfoTagController {
    @Autowired
    private IModelVideoInfoTagService modelVideoInfoTagService;

    /**
     * 分页列表查询
     *
     * @param modelVideoInfoTag
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "模板视频分类信息-分页列表查询")
    @ApiOperation(value = "模板视频分类信息-分页列表查询", notes = "模板视频分类信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ModelVideoInfoTag>> queryPageList(ModelVideoInfoTag modelVideoInfoTag,
                                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                          HttpServletRequest req) {
        Result<IPage<ModelVideoInfoTag>> result = new Result<IPage<ModelVideoInfoTag>>();
        QueryWrapper<ModelVideoInfoTag> queryWrapper = QueryGenerator.initQueryWrapper(modelVideoInfoTag, req.getParameterMap());
        Page<ModelVideoInfoTag> page = new Page<ModelVideoInfoTag>(pageNo, pageSize);
        IPage<ModelVideoInfoTag> pageList = modelVideoInfoTagService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param modelVideoInfoTag
     * @return
     */
    @AutoLog(value = "模板视频分类信息-添加")
    @ApiOperation(value = "模板视频分类信息-添加", notes = "模板视频分类信息-添加")
    @PostMapping(value = "/add")
    public Result<ModelVideoInfoTag> add(@RequestBody ModelVideoInfoTag modelVideoInfoTag) {
        Result<ModelVideoInfoTag> result = new Result<ModelVideoInfoTag>();
        try {
            modelVideoInfoTagService.save(modelVideoInfoTag);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param modelVideoInfoTag
     * @return
     */
    @AutoLog(value = "模板视频分类信息-编辑")
    @ApiOperation(value = "模板视频分类信息-编辑", notes = "模板视频分类信息-编辑")
    @PutMapping(value = "/edit")
    public Result<ModelVideoInfoTag> edit(@RequestBody ModelVideoInfoTag modelVideoInfoTag) {
        Result<ModelVideoInfoTag> result = new Result<ModelVideoInfoTag>();
        ModelVideoInfoTag modelVideoInfoTagEntity = modelVideoInfoTagService.getById(modelVideoInfoTag.getId());
        if (modelVideoInfoTagEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = modelVideoInfoTagService.updateById(modelVideoInfoTag);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "模板视频分类信息-通过id删除")
    @ApiOperation(value = "模板视频分类信息-通过id删除", notes = "模板视频分类信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            modelVideoInfoTagService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "模板视频分类信息-批量删除")
    @ApiOperation(value = "模板视频分类信息-批量删除", notes = "模板视频分类信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ModelVideoInfoTag> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ModelVideoInfoTag> result = new Result<ModelVideoInfoTag>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.modelVideoInfoTagService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "模板视频分类信息-通过id查询")
    @ApiOperation(value = "模板视频分类信息-通过id查询", notes = "模板视频分类信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ModelVideoInfoTag> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ModelVideoInfoTag> result = new Result<ModelVideoInfoTag>();
        ModelVideoInfoTag modelVideoInfoTag = modelVideoInfoTagService.getById(id);
        if (modelVideoInfoTag == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(modelVideoInfoTag);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<ModelVideoInfoTag> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                ModelVideoInfoTag modelVideoInfoTag = JSON.parseObject(deString, ModelVideoInfoTag.class);
                queryWrapper = QueryGenerator.initQueryWrapper(modelVideoInfoTag, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<ModelVideoInfoTag> pageList = modelVideoInfoTagService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "模板视频分类信息列表");
        mv.addObject(NormalExcelConstants.CLASS, ModelVideoInfoTag.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("模板视频分类信息列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<ModelVideoInfoTag> listModelVideoInfoTags = ExcelImportUtil.importExcel(file.getInputStream(), ModelVideoInfoTag.class, params);
                modelVideoInfoTagService.saveBatch(listModelVideoInfoTags);
                return Result.ok("文件导入成功！数据行数:" + listModelVideoInfoTags.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
