package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.CtopAdConstant;
import cn.com.ctop.toutiao.modules.report.service.IByteDanceHourlyAccountWarningService;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.log.XxlJobLogger;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 头条预警
 *
 * @author yumeng
 */
@Component
public class ByteDanceHourlyAccountWarningLoadJob {
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private IByteDanceHourlyAccountWarningService byteDanceHourlyAccountWarningService;
    static ExecutorService executorService = null;

    @XxlJob("byteDanceHourlyAccountWarning")
    public ReturnT<String> execute(String param) throws Exception {
        XxlJobLogger.log("开始时段消耗预警定时任务");
        long startTime = System.currentTimeMillis();
        List<CtopOauthToken> tokens = tokenService.getTokenListByType(CtopAdConstant.PLATFORM_TYPE_BYTEDANCE);
        if (null == tokens || tokens.size() <= 0) {
            XxlJobLogger.log("定时获取头条数据异常：未获取到可用的token");
            return ReturnT.FAIL;
        }
        executorService = Executors.newFixedThreadPool(3);
        String statDate = DateUtils.getDate("yyyy-MM-dd");
        String anotherDay = DateUtils.getAnotherDay("yyyy-MM-dd", statDate, 1);
        tokens.forEach(token -> executorService.submit(() -> byteDanceHourlyAccountWarningService.byteDanceAccountWarning(token.getAccountId(), statDate, anotherDay)));
        XxlJobLogger.log("时段消耗预警定时任务执行完毕，耗时：{} ms", System.currentTimeMillis() - startTime);
        return ReturnT.SUCCESS;
    }
}
