package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.CtopAdConstant;
import cn.com.ctop.toutiao.modules.report.service.IReportService;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.log.XxlJobLogger;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @author 宋英豪
 * 头条广告主时报获取任务
 */
@Component
public class BytedanceAdvertiserHourlyReportLoadJob {
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private IReportService reportService;
    static ExecutorService executorService = null;

    @XxlJob("bytedanceAdvertiserHourlyReport")
    public ReturnT<String> execute(String param) throws Exception {
        XxlJobLogger.log("头条时报数据获取开始");
        Date getDate = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH");
        String hour = simpleDateFormat.format(getDate);
        //1:查询当日数据
        List<CtopOauthToken> tokens = tokenService.getTokenListByType(CtopAdConstant.PLATFORM_TYPE_BYTEDANCE);
        if (null == tokens || tokens.size() <= 0) {
            XxlJobLogger.log("定时获取头条小时数据异常：未获取到可用的token");
            return ReturnT.FAIL;
        }
        executorService = Executors.newFixedThreadPool(4);
        tokens.forEach(token -> {
            executorService.submit(() -> {
                if (null != hour && "00".equals(hour)) {
                    Date finalGetDate = DateUtils.addDay(getDate, -1);
                    //获取广告主信息数据
                    reportService.getAdvertiserReport(token, finalGetDate, finalGetDate, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY);
                }
                reportService.getAdvertiserReport(token, getDate, getDate, CtopAdConstant.BYTEDANCE_REPORT_TYPE_HOURLY);
            });
        });
        XxlJobLogger.log("头条时报数据获取结束");
        return ReturnT.SUCCESS;
    }
}
