package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouCampaignService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouInterfaceService;
import com.alibaba.fastjson.JSONArray;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.log.XxlJobLogger;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 获取创意分天报表
 *
 * @author jeecg-boot
 */
@Component
public class KuaishouDailyCreativeReportLoadJob {
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private IKuaishouInterfaceService kuaishouInterfaceService;

    static ExecutorService executorService = null;

    @Autowired
    private IKuaiShouCampaignService campaignService;

    @XxlJob("kuaishouDailyCreativeReport")
    public ReturnT<String> execute(String param) throws Exception {
        Date getDate = DateUtils.addDay(new Date(), -1);
        executorService = Executors.newFixedThreadPool(4);
        List<CtopOauthToken> tokens = tokenService.selectKuaiShouToken();
        if (null == tokens || tokens.size() <= 0) {
            XxlJobLogger.log("定时获取快手数据异常：未获取到可用的token");
            return ReturnT.FAIL;
        }
        tokens.forEach(token -> {
            XxlJobLogger.log("获取创意分天报表开始,accountId:{}", token.getAccountId());
            kuaishouInterfaceService.getCampaignList(token, null, null);
            try {
                Thread.sleep(5 * 100);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            List<JSONArray> campaigns = campaignService.getJsonArrByAccount(token.getAccountId(), 50);
            if (!Check.isNull(campaigns) && campaigns.size() > 0) {
                campaigns.forEach(campaign -> executorService.submit(() -> kuaishouInterfaceService.getAdvertiserCreativeReportDaily(token, getDate, getDate, campaign)));
            }
        });
        return ReturnT.SUCCESS;
    }

}
