package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouDailyReportTaskService;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.log.XxlJobLogger;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import static org.jeecg.common.util.DateUtils.getAnotherDay;

/**
 * 获取文件状态并下载入库
 *
 * @author yumeng
 */
@Component
public class KuaishouDailyLoadFileJob {
    @Autowired
    private ICtopOauthTokenService tokenService;
    private ExecutorService executorService = null;
    @Autowired
    private IKuaiShouDailyReportTaskService dailyReportTaskService;

    @XxlJob("kuaishouDailyLoadFile")
    public ReturnT<String> execute(String param) throws Exception {
        String nowDate = DateUtils.getDate("yyyy-MM-dd");
        String statDate = getAnotherDay("yyyy-MM-dd", nowDate, -1);
        //  查询快手token
        List<CtopOauthToken> tokens = tokenService.selectKuaiShouToken();
        if (null == tokens || tokens.size() <= 0) {
            XxlJobLogger.log("定时获取快手数据异常：未获取到可用的token");
            return ReturnT.FAIL;
        }
        executorService = Executors.newFixedThreadPool(3);
        tokens.forEach(token -> executorService.submit(() -> dailyReportTaskService.getTaskList(token.getAccountId(), token.getAccessToken(), statDate)));
        return ReturnT.SUCCESS;
    }
}
