package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.kuaishou.modules.batch.service.IAccountWarningService;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.log.XxlJobLogger;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import static org.jeecg.common.util.DateUtils.getAnotherDay;

/**
 *
 * @author 于蒙
 */
@Component
public class KuaishouHourlyAccountWarningLoadJob {
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private IAccountWarningService accountWarningService;
    static ExecutorService executorService = null;

    @XxlJob("kuaishouHourlyAccountWarning")
    public ReturnT<String> execute(String param) throws Exception {
        XxlJobLogger.log("开始时段消耗预警定时任务");
        long l = System.currentTimeMillis();
        List<CtopOauthToken> tokens = tokenService.selectKuaiShouToken();
        if (null == tokens || tokens.size() <= 0) {
            XxlJobLogger.log("定时获取快手数据异常：未获取到可用的token");
            return ReturnT.FAIL;
        }
        String statDate = DateUtils.getDate("yyyy-MM-dd");
        String anotherDay = getAnotherDay("yyyy-MM-dd", statDate, -1);
        executorService = Executors.newFixedThreadPool(3);
        tokens.forEach(token -> executorService.submit(() -> accountWarningService.kuaiShouAccountWarning(token.getAccountId(), statDate, anotherDay)));
        XxlJobLogger.log("异步时段消耗预警定时任务执行完毕，耗时：{} ms", System.currentTimeMillis() - l);
        return ReturnT.SUCCESS;
    }
}
