package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouDirectionalTemplate;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouDirectionalTemplateService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 快手-定向模板
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-01-07
 */
@Slf4j
@Api(tags = "快手-定向模板")
@RestController
@RequestMapping("/test/kuaiShouDirectionalTemplate")
public class KuaiShouDirectionalTemplateController {
    @Autowired
    private IKuaiShouDirectionalTemplateService kuaiShouDirectionalTemplateService;






    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "快手-定向模板-批量删除")
    @ApiOperation(value = "快手-定向模板-批量删除", notes = "快手-定向模板-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouDirectionalTemplate> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaiShouDirectionalTemplate> result = new Result<KuaiShouDirectionalTemplate>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouDirectionalTemplateService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-定向模板-通过id查询")
    @ApiOperation(value = "快手-定向模板-通过id查询", notes = "快手-定向模板-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouDirectionalTemplate> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouDirectionalTemplate> result = new Result<KuaiShouDirectionalTemplate>();
        KuaiShouDirectionalTemplate kuaiShouDirectionalTemplate = kuaiShouDirectionalTemplateService.getById(id);
        if (kuaiShouDirectionalTemplate == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouDirectionalTemplate);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaiShouDirectionalTemplate> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaiShouDirectionalTemplate kuaiShouDirectionalTemplate = JSON.parseObject(deString, KuaiShouDirectionalTemplate.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouDirectionalTemplate, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaiShouDirectionalTemplate> pageList = kuaiShouDirectionalTemplateService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "快手-定向模板列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaiShouDirectionalTemplate.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("快手-定向模板列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaiShouDirectionalTemplate> listKuaiShouDirectionalTemplates = ExcelImportUtil.importExcel(file.getInputStream(), KuaiShouDirectionalTemplate.class, params);
                kuaiShouDirectionalTemplateService.saveBatch(listKuaiShouDirectionalTemplates);
                return Result.ok("文件导入成功！数据行数:" + listKuaiShouDirectionalTemplates.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
