package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouHistoryReportTask;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouHistoryReportTaskService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 快手历史数据任务记录
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-03-09
 */
@Slf4j
@Api(tags = "快手历史数据任务记录")
@RestController
@RequestMapping("/ctop/kuaiShouReportTask")
public class KuaiShouReportTaskController {
    @Autowired
    private IKuaiShouHistoryReportTaskService kuaiShouReportTaskService;

    /**
     * 分页列表查询
     *
     * @param kuaiShouReportTask
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "快手历史数据任务记录-分页列表查询")
    @ApiOperation(value = "快手历史数据任务记录-分页列表查询", notes = "快手历史数据任务记录-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaiShouHistoryReportTask>> queryPageList(KuaiShouHistoryReportTask kuaiShouReportTask,
                                                                  @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                  HttpServletRequest req) {
        Result<IPage<KuaiShouHistoryReportTask>> result = new Result<IPage<KuaiShouHistoryReportTask>>();
        QueryWrapper<KuaiShouHistoryReportTask> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouReportTask, req.getParameterMap());
        Page<KuaiShouHistoryReportTask> page = new Page<KuaiShouHistoryReportTask>(pageNo, pageSize);
        IPage<KuaiShouHistoryReportTask> pageList = kuaiShouReportTaskService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaiShouReportTask
     * @return
     */
    @AutoLog(value = "快手历史数据任务记录-添加")
    @ApiOperation(value = "快手历史数据任务记录-添加", notes = "快手历史数据任务记录-添加")
    @PostMapping(value = "/add")
    public Result<KuaiShouHistoryReportTask> add(@RequestBody KuaiShouHistoryReportTask kuaiShouReportTask) {
        Result<KuaiShouHistoryReportTask> result = new Result<KuaiShouHistoryReportTask>();
        try {
            kuaiShouReportTaskService.save(kuaiShouReportTask);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaiShouReportTask
     * @return
     */
    @AutoLog(value = "快手历史数据任务记录-编辑")
    @ApiOperation(value = "快手历史数据任务记录-编辑", notes = "快手历史数据任务记录-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaiShouHistoryReportTask> edit(@RequestBody KuaiShouHistoryReportTask kuaiShouReportTask) {
        Result<KuaiShouHistoryReportTask> result = new Result<KuaiShouHistoryReportTask>();
        KuaiShouHistoryReportTask kuaiShouReportTaskEntity = kuaiShouReportTaskService.getById(kuaiShouReportTask.getId());
        if (kuaiShouReportTaskEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaiShouReportTaskService.updateById(kuaiShouReportTask);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手历史数据任务记录-通过id删除")
    @ApiOperation(value = "快手历史数据任务记录-通过id删除", notes = "快手历史数据任务记录-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaiShouReportTaskService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "快手历史数据任务记录-批量删除")
    @ApiOperation(value = "快手历史数据任务记录-批量删除", notes = "快手历史数据任务记录-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouHistoryReportTask> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaiShouHistoryReportTask> result = new Result<KuaiShouHistoryReportTask>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouReportTaskService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手历史数据任务记录-通过id查询")
    @ApiOperation(value = "快手历史数据任务记录-通过id查询", notes = "快手历史数据任务记录-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouHistoryReportTask> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouHistoryReportTask> result = new Result<KuaiShouHistoryReportTask>();
        KuaiShouHistoryReportTask kuaiShouReportTask = kuaiShouReportTaskService.getById(id);
        if (kuaiShouReportTask == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouReportTask);
            result.setSuccess(true);
        }
        return result;
    }
}
