package cn.com.ctop.toutiao.modules.material.controller;

import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.modules.material.entity.ByteDanceCampaign;
import cn.com.ctop.toutiao.modules.material.service.IByteDanceCampaignService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * @Description: 今日头条广告组信息
 * @Author: jeecg-boot
 * @Date: 2019-07-22
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "今日头条广告组信息")
@RestController
@RequestMapping("/ctop/byteDanceCampaign")
public class ByteDanceCampaignController {
    @Resource
    private IByteDanceCampaignService byteDanceCampaignService;

    /**
     * 分页列表查询
     *
     * @param byteDanceCampaign
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "今日头条广告组信息-分页列表查询")
    @ApiOperation(value = "今日头条广告组信息-分页列表查询", notes = "今日头条广告组信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ByteDanceCampaign>> queryPageList(ByteDanceCampaign byteDanceCampaign,
                                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                          HttpServletRequest req) {
        Result<IPage<ByteDanceCampaign>> result = new Result<>();
        QueryWrapper<ByteDanceCampaign> queryWrapper = QueryGenerator.initQueryWrapper(byteDanceCampaign, req.getParameterMap());
        Page<ByteDanceCampaign> page = new Page<>(pageNo, pageSize);
        IPage<ByteDanceCampaign> pageList = byteDanceCampaignService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param byteDanceCampaign
     * @return
     */
    @AutoLog(value = "今日头条广告组信息-添加")
    @ApiOperation(value = "今日头条广告组信息-添加", notes = "今日头条广告组信息-添加")
    @PostMapping(value = "/add")
    public Result<ByteDanceCampaign> add(@RequestBody ByteDanceCampaign byteDanceCampaign) {
        Result<ByteDanceCampaign> result = new Result<>();
        try {
            byteDanceCampaignService.save(byteDanceCampaign);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param byteDanceCampaign
     * @return
     */
    @AutoLog(value = "今日头条广告组信息-编辑")
    @ApiOperation(value = "今日头条广告组信息-编辑", notes = "今日头条广告组信息-编辑")
    @PutMapping(value = "/edit")
    public Result<ByteDanceCampaign> edit(@RequestBody ByteDanceCampaign byteDanceCampaign) {
        Result<ByteDanceCampaign> result = new Result<>();
        ByteDanceCampaign byteDanceCampaignEntity = byteDanceCampaignService.getById(byteDanceCampaign.getId());
        if (byteDanceCampaignEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = byteDanceCampaignService.updateById(byteDanceCampaign);
            if (ok) {
                result.success("修改成功!");
            }else {
                result.error500("未知错误");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条广告组信息-通过id删除")
    @ApiOperation(value = "今日头条广告组信息-通过id删除", notes = "今日头条广告组信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            byteDanceCampaignService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "今日头条广告组信息-批量删除")
    @ApiOperation(value = "今日头条广告组信息-批量删除", notes = "今日头条广告组信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ByteDanceCampaign> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ByteDanceCampaign> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.byteDanceCampaignService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条广告组信息-通过id查询")
    @ApiOperation(value = "今日头条广告组信息-通过id查询", notes = "今日头条广告组信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ByteDanceCampaign> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ByteDanceCampaign> result = new Result<>();
        ByteDanceCampaign byteDanceCampaign = byteDanceCampaignService.getById(id);
        if (byteDanceCampaign == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(byteDanceCampaign);
            result.setSuccess(true);
        }
        return result;
    }
}
