package cn.com.ctop.toutiao.modules.material.controller;

import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.modules.material.entity.ByteDanceCreative;
import cn.com.ctop.toutiao.modules.material.service.IByteDanceCreativeService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * @Description: 今日头条创意信息
 * @Author: jeecg-boot
 * @Date: 2019-07-22
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "今日头条创意信息")
@RestController
@RequestMapping("/ctop/byteDanceCreative")
public class ByteDanceCreativeController {
    @Autowired
    private IByteDanceCreativeService byteDanceCreativeService;

    /**
     * 分页列表查询
     *
     * @param byteDanceCreative
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "今日头条创意信息-分页列表查询")
    @ApiOperation(value = "今日头条创意信息-分页列表查询", notes = "今日头条创意信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ByteDanceCreative>> queryPageList(ByteDanceCreative byteDanceCreative,
                                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                          HttpServletRequest req) {
        Result<IPage<ByteDanceCreative>> result = new Result<>();
        QueryWrapper<ByteDanceCreative> queryWrapper = QueryGenerator.initQueryWrapper(byteDanceCreative, req.getParameterMap());
        Page<ByteDanceCreative> page = new Page<>(pageNo, pageSize);
        IPage<ByteDanceCreative> pageList = byteDanceCreativeService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param byteDanceCreative
     * @return
     */
    @AutoLog(value = "今日头条创意信息-添加")
    @ApiOperation(value = "今日头条创意信息-添加", notes = "今日头条创意信息-添加")
    @PostMapping(value = "/add")
    public Result<ByteDanceCreative> add(@RequestBody ByteDanceCreative byteDanceCreative) {
        Result<ByteDanceCreative> result = new Result<>();
        try {
            byteDanceCreativeService.save(byteDanceCreative);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param byteDanceCreative
     * @return
     */
    @AutoLog(value = "今日头条创意信息-编辑")
    @ApiOperation(value = "今日头条创意信息-编辑", notes = "今日头条创意信息-编辑")
    @PutMapping(value = "/edit")
    public Result<ByteDanceCreative> edit(@RequestBody ByteDanceCreative byteDanceCreative) {
        Result<ByteDanceCreative> result = new Result<>();
        ByteDanceCreative byteDanceCreativeEntity = byteDanceCreativeService.getById(byteDanceCreative.getId());
        if (byteDanceCreativeEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = byteDanceCreativeService.updateById(byteDanceCreative);
            if (ok) {
                result.success("修改成功!");
            } else {
                result.error500("未知错误");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条创意信息-通过id删除")
    @ApiOperation(value = "今日头条创意信息-通过id删除", notes = "今日头条创意信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            byteDanceCreativeService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "今日头条创意信息-批量删除")
    @ApiOperation(value = "今日头条创意信息-批量删除", notes = "今日头条创意信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ByteDanceCreative> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ByteDanceCreative> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.byteDanceCreativeService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条创意信息-通过id查询")
    @ApiOperation(value = "今日头条创意信息-通过id查询", notes = "今日头条创意信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ByteDanceCreative> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ByteDanceCreative> result = new Result<>();
        ByteDanceCreative byteDanceCreative = byteDanceCreativeService.getById(id);
        if (byteDanceCreative == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(byteDanceCreative);
            result.setSuccess(true);
        }
        return result;
    }
}
