package cn.com.ctop.toutiao.modules.report.controller;

import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.modules.report.entity.BytedanceAdvertiserDailyReport;
import cn.com.ctop.toutiao.modules.report.service.IBytedanceAdvertiserDailyReportService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * @Description: 广告主日报表
 * @Author: jeecg-boot
 * @Date: 2019-09-10
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "广告主日报表")
@RestController
@RequestMapping("/ctop/bytedanceAdvertiserDailyReport")
public class BytedanceAdvertiserDailyReportController {
    @Autowired
    private IBytedanceAdvertiserDailyReportService bytedanceAdvertiserDailyReportService;

    /**
     * 分页列表查询
     *
     * @param bytedanceAdvertiserDailyReport
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "广告主日报表-分页列表查询")
    @ApiOperation(value = "广告主日报表-分页列表查询", notes = "广告主日报表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<BytedanceAdvertiserDailyReport>> queryPageList(BytedanceAdvertiserDailyReport bytedanceAdvertiserDailyReport,
                                                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                       HttpServletRequest req) {
        Result<IPage<BytedanceAdvertiserDailyReport>> result = new Result<>();
        QueryWrapper<BytedanceAdvertiserDailyReport> queryWrapper = QueryGenerator.initQueryWrapper(bytedanceAdvertiserDailyReport, req.getParameterMap());
        Page<BytedanceAdvertiserDailyReport> page = new Page<>(pageNo, pageSize);
        IPage<BytedanceAdvertiserDailyReport> pageList = bytedanceAdvertiserDailyReportService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param bytedanceAdvertiserDailyReport
     * @return
     */
    @AutoLog(value = "广告主日报表-添加")
    @ApiOperation(value = "广告主日报表-添加", notes = "广告主日报表-添加")
    @PostMapping(value = "/add")
    public Result<BytedanceAdvertiserDailyReport> add(@RequestBody BytedanceAdvertiserDailyReport bytedanceAdvertiserDailyReport) {
        Result<BytedanceAdvertiserDailyReport> result = new Result<>();
        try {
            bytedanceAdvertiserDailyReportService.save(bytedanceAdvertiserDailyReport);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param bytedanceAdvertiserDailyReport
     * @return
     */
    @AutoLog(value = "广告主日报表-编辑")
    @ApiOperation(value = "广告主日报表-编辑", notes = "广告主日报表-编辑")
    @PutMapping(value = "/edit")
    public Result<BytedanceAdvertiserDailyReport> edit(@RequestBody BytedanceAdvertiserDailyReport bytedanceAdvertiserDailyReport) {
        Result<BytedanceAdvertiserDailyReport> result = new Result<>();
        BytedanceAdvertiserDailyReport bytedanceAdvertiserDailyReportEntity = bytedanceAdvertiserDailyReportService.getById(bytedanceAdvertiserDailyReport.getId());
        if (bytedanceAdvertiserDailyReportEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = bytedanceAdvertiserDailyReportService.updateById(bytedanceAdvertiserDailyReport);
            if (ok) {
                result.success("修改成功!");
            }else {
                result.error500("未知错误");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告主日报表-通过id删除")
    @ApiOperation(value = "广告主日报表-通过id删除", notes = "广告主日报表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            bytedanceAdvertiserDailyReportService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "广告主日报表-批量删除")
    @ApiOperation(value = "广告主日报表-批量删除", notes = "广告主日报表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<BytedanceAdvertiserDailyReport> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<BytedanceAdvertiserDailyReport> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.bytedanceAdvertiserDailyReportService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告主日报表-通过id查询")
    @ApiOperation(value = "广告主日报表-通过id查询", notes = "广告主日报表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<BytedanceAdvertiserDailyReport> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<BytedanceAdvertiserDailyReport> result = new Result<>();
        BytedanceAdvertiserDailyReport bytedanceAdvertiserDailyReport = bytedanceAdvertiserDailyReportService.getById(id);
        if (bytedanceAdvertiserDailyReport == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(bytedanceAdvertiserDailyReport);
            result.setSuccess(true);
        }
        return result;
    }
}
