package cn.com.ctop.toutiao.modules.report.controller;

import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.modules.report.entity.BytedanceDailyReportTask;
import cn.com.ctop.toutiao.modules.report.service.IBytedanceDailyReportTaskService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 头条异步报表任务表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-04-16
 */
@Slf4j
@Api(tags="头条异步报表任务表")
@RestController
@RequestMapping("/bytedance/bytedanceDailyReportTask")
public class BytedanceDailyReportTaskController {
    @Autowired
    private IBytedanceDailyReportTaskService bytedanceDailyReportTaskService;

    /**
     * 分页列表查询
     *
     * @param bytedanceDailyReportTask
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "头条异步报表任务表-分页列表查询")
    @ApiOperation(value = "头条异步报表任务表-分页列表查询", notes = "头条异步报表任务表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<BytedanceDailyReportTask>> queryPageList(BytedanceDailyReportTask bytedanceDailyReportTask,
                                                                 @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                 @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                 HttpServletRequest req) {
        Result<IPage<BytedanceDailyReportTask>> result = new Result<>();
        QueryWrapper<BytedanceDailyReportTask> queryWrapper = QueryGenerator.initQueryWrapper(bytedanceDailyReportTask, req.getParameterMap());
        Page<BytedanceDailyReportTask> page = new Page<>(pageNo, pageSize);
        IPage<BytedanceDailyReportTask> pageList = bytedanceDailyReportTaskService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param bytedanceDailyReportTask
     * @return
     */
    @AutoLog(value = "头条异步报表任务表-添加")
    @ApiOperation(value = "头条异步报表任务表-添加", notes = "头条异步报表任务表-添加")
    @PostMapping(value = "/add")
    public Result<BytedanceDailyReportTask> add(@RequestBody BytedanceDailyReportTask bytedanceDailyReportTask) {
        Result<BytedanceDailyReportTask> result = new Result<>();
        try {
            bytedanceDailyReportTaskService.save(bytedanceDailyReportTask);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param bytedanceDailyReportTask
     * @return
     */
    @AutoLog(value = "头条异步报表任务表-编辑")
    @ApiOperation(value = "头条异步报表任务表-编辑", notes = "头条异步报表任务表-编辑")
    @PutMapping(value = "/edit")
    public Result<BytedanceDailyReportTask> edit(@RequestBody BytedanceDailyReportTask bytedanceDailyReportTask) {
        Result<BytedanceDailyReportTask> result = new Result<>();
        BytedanceDailyReportTask bytedanceDailyReportTaskEntity = bytedanceDailyReportTaskService.getById(bytedanceDailyReportTask.getId());
        if (bytedanceDailyReportTaskEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = bytedanceDailyReportTaskService.updateById(bytedanceDailyReportTask);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "头条异步报表任务表-通过id删除")
    @ApiOperation(value = "头条异步报表任务表-通过id删除", notes = "头条异步报表任务表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            bytedanceDailyReportTaskService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "头条异步报表任务表-批量删除")
    @ApiOperation(value = "头条异步报表任务表-批量删除", notes = "头条异步报表任务表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<BytedanceDailyReportTask> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<BytedanceDailyReportTask> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.bytedanceDailyReportTaskService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "头条异步报表任务表-通过id查询")
    @ApiOperation(value = "头条异步报表任务表-通过id查询", notes = "头条异步报表任务表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<BytedanceDailyReportTask> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<BytedanceDailyReportTask> result = new Result<>();
        BytedanceDailyReportTask bytedanceDailyReportTask = bytedanceDailyReportTaskService.getById(id);
        if (bytedanceDailyReportTask == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(bytedanceDailyReportTask);
            result.setSuccess(true);
        }
        return result;
    }
}
