package cn.com.ctop.toutiao.modules.report.controller;

import cn.com.ctop.toutiao.modules.report.entity.BytedanceReportMaterialDaily;
import cn.com.ctop.toutiao.modules.report.service.IBytedanceReportMaterialDailyService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 素材报表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-04-16
 */
@Slf4j
@Api(tags="素材报表")
@RestController
@RequestMapping("/bytedance/bytedanceReportMaterialDaily")
public class BytedanceReportMaterialDailyController {
    @Autowired
    private IBytedanceReportMaterialDailyService bytedanceReportMaterialDailyService;

    /**
     * 分页列表查询
     *
     * @param bytedanceReportMaterialDaily
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "素材报表-分页列表查询")
    @ApiOperation(value = "素材报表-分页列表查询", notes = "素材报表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<BytedanceReportMaterialDaily>> queryPageList(BytedanceReportMaterialDaily bytedanceReportMaterialDaily,
                                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                     HttpServletRequest req) {
        Result<IPage<BytedanceReportMaterialDaily>> result = new Result<>();
        QueryWrapper<BytedanceReportMaterialDaily> queryWrapper = QueryGenerator.initQueryWrapper(bytedanceReportMaterialDaily, req.getParameterMap());
        Page<BytedanceReportMaterialDaily> page = new Page<>(pageNo, pageSize);
        IPage<BytedanceReportMaterialDaily> pageList = bytedanceReportMaterialDailyService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param bytedanceReportMaterialDaily
     * @return
     */
    @AutoLog(value = "素材报表-添加")
    @ApiOperation(value = "素材报表-添加", notes = "素材报表-添加")
    @PostMapping(value = "/add")
    public Result<BytedanceReportMaterialDaily> add(@RequestBody BytedanceReportMaterialDaily bytedanceReportMaterialDaily) {
        Result<BytedanceReportMaterialDaily> result = new Result<>();
        try {
            bytedanceReportMaterialDailyService.save(bytedanceReportMaterialDaily);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param bytedanceReportMaterialDaily
     * @return
     */
    @AutoLog(value = "素材报表-编辑")
    @ApiOperation(value = "素材报表-编辑", notes = "素材报表-编辑")
    @PutMapping(value = "/edit")
    public Result<BytedanceReportMaterialDaily> edit(@RequestBody BytedanceReportMaterialDaily bytedanceReportMaterialDaily) {
        Result<BytedanceReportMaterialDaily> result = new Result<>();
        BytedanceReportMaterialDaily bytedanceReportMaterialDailyEntity = bytedanceReportMaterialDailyService.getById(bytedanceReportMaterialDaily.getId());
        if (bytedanceReportMaterialDailyEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = bytedanceReportMaterialDailyService.updateById(bytedanceReportMaterialDaily);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "素材报表-通过id删除")
    @ApiOperation(value = "素材报表-通过id删除", notes = "素材报表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            bytedanceReportMaterialDailyService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "素材报表-批量删除")
    @ApiOperation(value = "素材报表-批量删除", notes = "素材报表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<BytedanceReportMaterialDaily> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<BytedanceReportMaterialDaily> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.bytedanceReportMaterialDailyService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "素材报表-通过id查询")
    @ApiOperation(value = "素材报表-通过id查询", notes = "素材报表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<BytedanceReportMaterialDaily> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<BytedanceReportMaterialDaily> result = new Result<>();
        BytedanceReportMaterialDaily bytedanceReportMaterialDaily = bytedanceReportMaterialDailyService.getById(id);
        if (bytedanceReportMaterialDaily == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(bytedanceReportMaterialDaily);
            result.setSuccess(true);
        }
        return result;
    }
}
