package cn.com.ctop.huananvideoreport.controller;

import java.text.ParseException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.com.ctop.userefficientvideomap.entity.PerformanceTimeDTO;
import cn.com.ctop.userefficientvideomap.entity.UserDto2;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.util.oConvertUtils;
import cn.com.ctop.huananvideoreport.entity.HuananVideoReport;
import cn.com.ctop.huananvideoreport.service.IHuananVideoReportService;

import java.util.Date;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 华南视频报表类
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-04-28
 */
@Slf4j
@Api(tags = "华南视频报表类")
@RestController
@RequestMapping("/huananvideoreport/huananVideoReport")
public class HuananVideoReportController {
    @Autowired
    private IHuananVideoReportService huananVideoReportService;

    /**
     * 分页列表查询
     *
     * @param huananVideoReport
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "华南视频报表类-分页列表查询")
    @ApiOperation(value = "华南视频报表类-分页列表查询", notes = "华南视频报表类-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<HuananVideoReport>> queryPageList(HuananVideoReport huananVideoReport,
                                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                          HttpServletRequest req) {
        Result<IPage<HuananVideoReport>> result = new Result<IPage<HuananVideoReport>>();
        QueryWrapper<HuananVideoReport> queryWrapper = QueryGenerator.initQueryWrapper(huananVideoReport, req.getParameterMap());
        Page<HuananVideoReport> page = new Page<HuananVideoReport>(pageNo, pageSize);
        IPage<HuananVideoReport> pageList = huananVideoReportService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param huananVideoReport
     * @return
     */
    @AutoLog(value = "华南视频报表类-添加")
    @ApiOperation(value = "华南视频报表类-添加", notes = "华南视频报表类-添加")
    @PostMapping(value = "/add")
    public Result<HuananVideoReport> add(@RequestBody HuananVideoReport huananVideoReport) {
        Result<HuananVideoReport> result = new Result<HuananVideoReport>();
        try {
            huananVideoReportService.save(huananVideoReport);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param huananVideoReport
     * @return
     */
    @AutoLog(value = "华南视频报表类-编辑")
    @ApiOperation(value = "华南视频报表类-编辑", notes = "华南视频报表类-编辑")
    @PutMapping(value = "/edit")
    public Result<HuananVideoReport> edit(@RequestBody HuananVideoReport huananVideoReport) {
        Result<HuananVideoReport> result = new Result<HuananVideoReport>();
        HuananVideoReport huananVideoReportEntity = huananVideoReportService.getById(huananVideoReport.getId());
        if (huananVideoReportEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = huananVideoReportService.updateById(huananVideoReport);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "华南视频报表类-通过id删除")
    @ApiOperation(value = "华南视频报表类-通过id删除", notes = "华南视频报表类-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            huananVideoReportService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "华南视频报表类-批量删除")
    @ApiOperation(value = "华南视频报表类-批量删除", notes = "华南视频报表类-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<HuananVideoReport> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<HuananVideoReport> result = new Result<HuananVideoReport>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.huananVideoReportService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "华南视频报表类-通过id查询")
    @ApiOperation(value = "华南视频报表类-通过id查询", notes = "华南视频报表类-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<HuananVideoReport> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<HuananVideoReport> result = new Result<HuananVideoReport>();
        HuananVideoReport huananVideoReport = huananVideoReportService.getById(id);
        if (huananVideoReport == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(huananVideoReport);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<HuananVideoReport> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                HuananVideoReport huananVideoReport = JSON.parseObject(deString, HuananVideoReport.class);
                queryWrapper = QueryGenerator.initQueryWrapper(huananVideoReport, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<HuananVideoReport> pageList = huananVideoReportService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "华南视频报表类列表");
        mv.addObject(NormalExcelConstants.CLASS, HuananVideoReport.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("华南视频报表类列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<HuananVideoReport> listHuananVideoReports = ExcelImportUtil.importExcel(file.getInputStream(), HuananVideoReport.class, params);
                huananVideoReportService.saveBatch(listHuananVideoReports);
                return Result.ok("文件导入成功！数据行数:" + listHuananVideoReports.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }


    @AutoLog(value = "头条优化季度绩效报表")
    @ApiOperation(value = "头条优化季度绩效报表", notes = "头条优化季度绩效报表")
    @PostMapping(value = "/getHuananMaterialInfo")
    public Result getHuananMaterialInfo(HttpServletRequest req) throws ParseException {
        Result<List<UserDto2>> result = new Result<List<UserDto2>>();
        huananVideoReportService.getHuananMaterialInfo("2020-05-01", "2020-05-07");
        result.setSuccess(true);
        return result;
    }

}
