package cn.com.ctop.planeprojectcost.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.util.oConvertUtils;
import cn.com.ctop.planeprojectcost.entity.PlaneProjectCost;
import cn.com.ctop.planeprojectcost.service.IPlaneProjectCostService;

import java.util.Date;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 平面设计项目消耗表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-12-28
 */
@Slf4j
@Api(tags = "平面设计项目消耗表")
@RestController
@RequestMapping("/planeprojectcost/planeProjectCost")
public class PlaneProjectCostController {
    @Autowired
    private IPlaneProjectCostService planeProjectCostService;

    /**
     * 分页列表查询
     *
     * @param planeProjectCost
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "平面设计项目消耗表-分页列表查询")
    @ApiOperation(value = "平面设计项目消耗表-分页列表查询", notes = "平面设计项目消耗表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<PlaneProjectCost>> queryPageList(PlaneProjectCost planeProjectCost,
                                                         @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                         @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                         HttpServletRequest req) {
        Result<IPage<PlaneProjectCost>> result = new Result<IPage<PlaneProjectCost>>();
        QueryWrapper<PlaneProjectCost> queryWrapper = QueryGenerator.initQueryWrapper(planeProjectCost, req.getParameterMap());
        Page<PlaneProjectCost> page = new Page<PlaneProjectCost>(pageNo, pageSize);
        IPage<PlaneProjectCost> pageList = planeProjectCostService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param planeProjectCost
     * @return
     */
    @AutoLog(value = "平面设计项目消耗表-添加")
    @ApiOperation(value = "平面设计项目消耗表-添加", notes = "平面设计项目消耗表-添加")
    @PostMapping(value = "/add")
    public Result<PlaneProjectCost> add(@RequestBody PlaneProjectCost planeProjectCost) {
        Result<PlaneProjectCost> result = new Result<PlaneProjectCost>();
        try {
            planeProjectCostService.save(planeProjectCost);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param planeProjectCost
     * @return
     */
    @AutoLog(value = "平面设计项目消耗表-编辑")
    @ApiOperation(value = "平面设计项目消耗表-编辑", notes = "平面设计项目消耗表-编辑")
    @PutMapping(value = "/edit")
    public Result<PlaneProjectCost> edit(@RequestBody PlaneProjectCost planeProjectCost) {
        Result<PlaneProjectCost> result = new Result<PlaneProjectCost>();
        PlaneProjectCost planeProjectCostEntity = planeProjectCostService.getById(planeProjectCost.getId());
        if (planeProjectCostEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = planeProjectCostService.updateById(planeProjectCost);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "平面设计项目消耗表-通过id删除")
    @ApiOperation(value = "平面设计项目消耗表-通过id删除", notes = "平面设计项目消耗表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            planeProjectCostService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "平面设计项目消耗表-批量删除")
    @ApiOperation(value = "平面设计项目消耗表-批量删除", notes = "平面设计项目消耗表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<PlaneProjectCost> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<PlaneProjectCost> result = new Result<PlaneProjectCost>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.planeProjectCostService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "平面设计项目消耗表-通过id查询")
    @ApiOperation(value = "平面设计项目消耗表-通过id查询", notes = "平面设计项目消耗表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<PlaneProjectCost> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<PlaneProjectCost> result = new Result<PlaneProjectCost>();
        PlaneProjectCost planeProjectCost = planeProjectCostService.getById(id);
        if (planeProjectCost == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(planeProjectCost);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<PlaneProjectCost> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                PlaneProjectCost planeProjectCost = JSON.parseObject(deString, PlaneProjectCost.class);
                queryWrapper = QueryGenerator.initQueryWrapper(planeProjectCost, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<PlaneProjectCost> pageList = planeProjectCostService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "平面设计项目消耗表列表");
        mv.addObject(NormalExcelConstants.CLASS, PlaneProjectCost.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("平面设计项目消耗表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<PlaneProjectCost> listPlaneProjectCosts = ExcelImportUtil.importExcel(file.getInputStream(), PlaneProjectCost.class, params);
                planeProjectCostService.saveBatch(listPlaneProjectCosts);
                return Result.ok("文件导入成功！数据行数:" + listPlaneProjectCosts.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
