package cn.com.ctop.uservideomap.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.util.oConvertUtils;
import cn.com.ctop.uservideomap.entity.UserVideoMap;
import cn.com.ctop.uservideomap.service.IUserVideoMapService;

import java.util.Date;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 视频和设计师对应关系表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-03-17
 */
@Slf4j
@Api(tags = "视频和设计师对应关系表")
@RestController
@RequestMapping("/uservideomap/userVideoMap")
public class UserVideoMapController {
    @Autowired
    private IUserVideoMapService userVideoMapService;

    /**
     * 分页列表查询
     *
     * @param userVideoMap
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "视频和设计师对应关系表-分页列表查询")
    @ApiOperation(value = "视频和设计师对应关系表-分页列表查询", notes = "视频和设计师对应关系表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<UserVideoMap>> queryPageList(UserVideoMap userVideoMap,
                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                     HttpServletRequest req) {
        Result<IPage<UserVideoMap>> result = new Result<IPage<UserVideoMap>>();
        QueryWrapper<UserVideoMap> queryWrapper = QueryGenerator.initQueryWrapper(userVideoMap, req.getParameterMap());
        Page<UserVideoMap> page = new Page<UserVideoMap>(pageNo, pageSize);
        IPage<UserVideoMap> pageList = userVideoMapService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param userVideoMap
     * @return
     */
    @AutoLog(value = "视频和设计师对应关系表-添加")
    @ApiOperation(value = "视频和设计师对应关系表-添加", notes = "视频和设计师对应关系表-添加")
    @PostMapping(value = "/add")
    public Result<UserVideoMap> add(@RequestBody UserVideoMap userVideoMap) {
        Result<UserVideoMap> result = new Result<UserVideoMap>();
        try {
            userVideoMapService.save(userVideoMap);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param userVideoMap
     * @return
     */
    @AutoLog(value = "视频和设计师对应关系表-编辑")
    @ApiOperation(value = "视频和设计师对应关系表-编辑", notes = "视频和设计师对应关系表-编辑")
    @PutMapping(value = "/edit")
    public Result<UserVideoMap> edit(@RequestBody UserVideoMap userVideoMap) {
        Result<UserVideoMap> result = new Result<UserVideoMap>();
        UserVideoMap userVideoMapEntity = userVideoMapService.getById(userVideoMap.getId());
        if (userVideoMapEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = userVideoMapService.updateById(userVideoMap);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "视频和设计师对应关系表-通过id删除")
    @ApiOperation(value = "视频和设计师对应关系表-通过id删除", notes = "视频和设计师对应关系表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            userVideoMapService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "视频和设计师对应关系表-批量删除")
    @ApiOperation(value = "视频和设计师对应关系表-批量删除", notes = "视频和设计师对应关系表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<UserVideoMap> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<UserVideoMap> result = new Result<UserVideoMap>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.userVideoMapService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "视频和设计师对应关系表-通过id查询")
    @ApiOperation(value = "视频和设计师对应关系表-通过id查询", notes = "视频和设计师对应关系表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<UserVideoMap> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<UserVideoMap> result = new Result<UserVideoMap>();
        UserVideoMap userVideoMap = userVideoMapService.getById(id);
        if (userVideoMap == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(userVideoMap);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<UserVideoMap> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                UserVideoMap userVideoMap = JSON.parseObject(deString, UserVideoMap.class);
                queryWrapper = QueryGenerator.initQueryWrapper(userVideoMap, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<UserVideoMap> pageList = userVideoMapService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "视频和设计师对应关系表列表");
        mv.addObject(NormalExcelConstants.CLASS, UserVideoMap.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("视频和设计师对应关系表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<UserVideoMap> listUserVideoMaps = ExcelImportUtil.importExcel(file.getInputStream(), UserVideoMap.class, params);
                userVideoMapService.saveBatch(listUserVideoMaps);
                return Result.ok("文件导入成功！数据行数:" + listUserVideoMaps.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
